@@ -4,11 +4,11 @@ import java.util.Enumeration;
 import java.util.Vector;
 
 /**
- * Mutable class for building ASN.1 constructed objects.
+ * Mutable class for building ASN.1 constructed objects such as SETs or SEQUENCEs.
  */
 public class ASN1EncodableVector
 {
-    Vector v = new Vector();
+    private final Vector v = new Vector();
 
     /**
      * Base constructor.
