@@ -6,9 +6,11 @@ import java.util.Vector;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.anssi.ANSSINamedCurves;
 import org.bouncycastle.asn1.cryptopro.ECGOST3410NamedCurves;
+import org.bouncycastle.asn1.gm.GMNamedCurves;
 import org.bouncycastle.asn1.nist.NISTNamedCurves;
 import org.bouncycastle.asn1.sec.SECNamedCurves;
 import org.bouncycastle.asn1.teletrust.TeleTrusTNamedCurves;
+import org.bouncycastle.crypto.params.ECDomainParameters;
 
 /**
  * A general class that reads all X9.62 style EC curve tables.
@@ -47,6 +49,16 @@ public class ECNamedCurveTable
             ecP = ANSSINamedCurves.getByName(name);
         }
 
+        if (ecP == null)
+        {
+            ecP = fromDomainParameters(ECGOST3410NamedCurves.getByName(name));
+        }
+
+        if (ecP == null)
+        {
+            ecP = GMNamedCurves.getByName(name);
+        }
+
         return ecP;
     }
 
@@ -81,6 +93,16 @@ public class ECNamedCurveTable
             oid = ANSSINamedCurves.getOID(name);
         }
 
+        if (oid == null)
+        {
+            oid = ECGOST3410NamedCurves.getOID(name);
+        }
+
+        if (oid == null)
+        {
+            oid = GMNamedCurves.getOID(name);
+        }
+
         return oid;
     }
 
@@ -94,13 +116,18 @@ public class ECNamedCurveTable
     public static String getName(
         ASN1ObjectIdentifier oid)
     {
-        String name = NISTNamedCurves.getName(oid);
+        String name = X962NamedCurves.getName(oid);
 
         if (name == null)
         {
             name = SECNamedCurves.getName(oid);
         }
 
+        if (name == null)
+        {
+            name = NISTNamedCurves.getName(oid);
+        }
+
         if (name == null)
         {
             name = TeleTrusTNamedCurves.getName(oid);
@@ -108,7 +135,7 @@ public class ECNamedCurveTable
 
         if (name == null)
         {
-            name = X962NamedCurves.getName(oid);
+            name = ANSSINamedCurves.getName(oid);
         }
 
         if (name == null)
@@ -116,6 +143,11 @@ public class ECNamedCurveTable
             name = ECGOST3410NamedCurves.getName(oid);
         }
 
+        if (name == null)
+        {
+            name = GMNamedCurves.getName(oid);
+        }
+
         return name;
     }
 
@@ -148,6 +180,16 @@ public class ECNamedCurveTable
             ecP = ANSSINamedCurves.getByOID(oid);
         }
 
+        if (ecP == null)
+        {
+            ecP = fromDomainParameters(ECGOST3410NamedCurves.getByOID(oid));
+        }
+
+        if (ecP == null)
+        {
+            ecP = GMNamedCurves.getByOID(oid);
+        }
+
         return ecP;
     }
 
@@ -165,6 +207,8 @@ public class ECNamedCurveTable
         addEnumeration(v, NISTNamedCurves.getNames());
         addEnumeration(v, TeleTrusTNamedCurves.getNames());
         addEnumeration(v, ANSSINamedCurves.getNames());
+        addEnumeration(v, ECGOST3410NamedCurves.getNames());
+        addEnumeration(v, GMNamedCurves.getNames());
 
         return v.elements();
     }
@@ -178,4 +222,9 @@ public class ECNamedCurveTable
             v.addElement(e.nextElement());
         }
     }
+
+    private static X9ECParameters fromDomainParameters(ECDomainParameters dp)
+    {
+        return dp == null ? null : new X9ECParameters(dp.getCurve(), dp.getG(), dp.getN(), dp.getH(), dp.getSeed());
+    }
 }
