@@ -4,21 +4,26 @@ import java.security.SecureRandom;
 
 import org.bouncycastle.crypto.KeyGenerationParameters;
 
-public class CramerShoupKeyGenerationParameters extends KeyGenerationParameters {
-	
+public class CramerShoupKeyGenerationParameters
+	extends KeyGenerationParameters
+{
+
 	private CramerShoupParameters params;
 
-	public CramerShoupKeyGenerationParameters(SecureRandom random, CramerShoupParameters params) {
+	public CramerShoupKeyGenerationParameters(SecureRandom random, CramerShoupParameters params)
+	{
 		super(random, getStrength(params));
 
 		this.params = params;
 	}
 
-	public CramerShoupParameters getParameters() {
+	public CramerShoupParameters getParameters()
+	{
 		return params;
 	}
 
-	static int getStrength(CramerShoupParameters params) {
+	static int getStrength(CramerShoupParameters params)
+	{
 		return params.getP().bitLength();
 	}
 }
