@@ -7,11 +7,6 @@ public class CompressionMethod
 {
     public static final short _null = 0;
 
-    /**
-     * @deprecated use '_null' instead
-     */
-    public static final short NULL = _null;
-
     /*
      * RFC 3749 2
      */
