@@ -28,4 +28,9 @@ public class ExporterLabel
      * RFC 5764
      */
     public static final String dtls_srtp = "EXTRACTOR-dtls_srtp";
+
+    /*
+     * draft-ietf-tls-session-hash-04
+     */
+    public static final String extended_master_secret = "extended master secret";
 }
