@@ -8,6 +8,7 @@ import org.bouncycastle.math.ec.endo.ECEndomorphism;
 import org.bouncycastle.math.ec.endo.GLVEndomorphism;
 import org.bouncycastle.math.field.FiniteField;
 import org.bouncycastle.math.field.FiniteFields;
+import org.bouncycastle.math.raw.Nat;
 import org.bouncycastle.util.BigIntegers;
 import org.bouncycastle.util.Integers;
 
@@ -75,9 +76,13 @@ public abstract class ECCurve
                 throw new IllegalStateException("implementation returned current curve");
             }
 
-            c.coord = coord;
-            c.endomorphism = endomorphism;
-            c.multiplier = multiplier;
+            // NOTE: Synchronization added to keep FindBugs™ happy
+            synchronized (c)
+            {
+                c.coord = coord;
+                c.endomorphism = endomorphism;
+                c.multiplier = multiplier;
+            }
 
             return c;
         }
@@ -100,7 +105,9 @@ public abstract class ECCurve
 
     public abstract ECFieldElement fromBigInteger(BigInteger x);
 
-    public Config configure()
+    public abstract boolean isValidFieldElement(BigInteger x);
+
+    public synchronized Config configure()
     {
         return new Config(this.coord, this.endomorphism, this.multiplier);
     }
@@ -229,26 +236,57 @@ public abstract class ECCurve
      */
     public void normalizeAll(ECPoint[] points)
     {
-        checkPoints(points);
+        normalizeAll(points, 0, points.length, null);
+    }
 
-        if (this.getCoordinateSystem() == ECCurve.COORD_AFFINE)
+    /**
+     * Normalization ensures that any projective coordinate is 1, and therefore that the x, y
+     * coordinates reflect those of the equivalent point in an affine coordinate system. Where more
+     * than one point is to be normalized, this method will generally be more efficient than
+     * normalizing each point separately. An (optional) z-scaling factor can be applied; effectively
+     * each z coordinate is scaled by this value prior to normalization (but only one
+     * actual multiplication is needed).
+     * 
+     * @param points
+     *            An array of points that will be updated in place with their normalized versions,
+     *            where necessary
+     * @param off
+     *            The start of the range of points to normalize
+     * @param len
+     *            The length of the range of points to normalize
+     * @param iso
+     *            The (optional) z-scaling factor - can be null
+     */
+    public void normalizeAll(ECPoint[] points, int off, int len, ECFieldElement iso)
+    {
+        checkPoints(points, off, len);
+
+        switch (this.getCoordinateSystem())
         {
+        case ECCurve.COORD_AFFINE:
+        case ECCurve.COORD_LAMBDA_AFFINE:
+        {
+            if (iso != null)
+            {
+                throw new IllegalArgumentException("'iso' not valid for affine coordinates");
+            }
             return;
         }
+        }
 
         /*
          * Figure out which of the points actually need to be normalized
          */
-        ECFieldElement[] zs = new ECFieldElement[points.length];
-        int[] indices = new int[points.length];
+        ECFieldElement[] zs = new ECFieldElement[len];
+        int[] indices = new int[len];
         int count = 0;
-        for (int i = 0; i < points.length; ++i)
+        for (int i = 0; i < len; ++i)
         {
-            ECPoint p = points[i];
-            if (null != p && !p.isNormalized())
+            ECPoint p = points[off + i];
+            if (null != p && (iso != null || !p.isNormalized()))
             {
                 zs[count] = p.getZCoord(0);
-                indices[count++] = i;
+                indices[count++] = off + i;
             }
         }
 
@@ -257,7 +295,7 @@ public abstract class ECCurve
             return;
         }
 
-        ECAlgorithms.montgomeryTrick(zs, 0, count);
+        ECAlgorithms.montgomeryTrick(zs, 0, count, iso);
 
         for (int j = 0; j < count; ++j)
         {
@@ -404,6 +442,61 @@ public abstract class ECCurve
         return p;
     }
 
+    /**
+     * Create a cache-safe lookup table for the specified sequence of points. All the points MUST
+     * belong to this {@link ECCurve} instance, and MUST already be normalized.
+     */
+    public ECLookupTable createCacheSafeLookupTable(final ECPoint[] points, int off, final int len)
+    {
+        final int FE_BYTES = (getFieldSize() + 7) >>> 3;
+
+        final byte[] table = new byte[len * FE_BYTES * 2];
+        {
+            int pos = 0;
+            for (int i = 0; i < len; ++i)
+            {
+                ECPoint p = points[off + i];
+                byte[] px = p.getRawXCoord().toBigInteger().toByteArray();
+                byte[] py = p.getRawYCoord().toBigInteger().toByteArray();
+
+                int pxStart = px.length > FE_BYTES ? 1 : 0, pxLen = px.length - pxStart;
+                int pyStart = py.length > FE_BYTES ? 1 : 0, pyLen = py.length - pyStart;
+
+                System.arraycopy(px, pxStart, table, pos + FE_BYTES - pxLen, pxLen); pos += FE_BYTES;
+                System.arraycopy(py, pyStart, table, pos + FE_BYTES - pyLen, pyLen); pos += FE_BYTES;
+            }
+        }
+
+        return new ECLookupTable()
+        {
+            public int getSize()
+            {
+                return len;
+            }
+
+            public ECPoint lookup(int index)
+            {
+                byte[] x = new byte[FE_BYTES], y = new byte[FE_BYTES];
+                int pos = 0;
+
+                for (int i = 0; i < len; ++i)
+                {
+                    int MASK = ((i ^ index) - 1) >> 31;
+
+                    for (int j = 0; j < FE_BYTES; ++j)
+                    {
+                        x[j] ^= table[pos + j] & MASK;
+                        y[j] ^= table[pos + FE_BYTES + j] & MASK;
+                    }
+
+                    pos += (FE_BYTES * 2);
+                }
+
+                return createRawPoint(fromBigInteger(new BigInteger(1, x)), fromBigInteger(new BigInteger(1, y)), false);
+            }
+        };
+    }
+
     protected void checkPoint(ECPoint point)
     {
         if (null == point || (this != point.getCurve()))
@@ -413,15 +506,24 @@ public abstract class ECCurve
     }
 
     protected void checkPoints(ECPoint[] points)
+    {
+        checkPoints(points, 0, points.length);
+    }
+
+    protected void checkPoints(ECPoint[] points, int off, int len)
     {
         if (points == null)
         {
             throw new IllegalArgumentException("'points' cannot be null");
         }
-
-        for (int i = 0; i < points.length; ++i)
+        if (off < 0 || len < 0 || (off > (points.length - len)))
         {
-            ECPoint point = points[i];
+            throw new IllegalArgumentException("invalid range specified for 'points'");
+        }
+
+        for (int i = 0; i < len; ++i)
+        {
+            ECPoint point = points[off + i];
             if (null != point && this != point.getCurve())
             {
                 throw new IllegalArgumentException("'points' entries must be null or on this curve");
@@ -457,10 +559,15 @@ public abstract class ECCurve
             super(FiniteFields.getPrimeField(q));
         }
 
+        public boolean isValidFieldElement(BigInteger x)
+        {
+            return x != null && x.signum() >= 0 && x.compareTo(this.getField().getCharacteristic()) < 0;
+        }
+
         protected ECPoint decompressPoint(int yTilde, BigInteger X1)
         {
             ECFieldElement x = this.fromBigInteger(X1);
-            ECFieldElement rhs = x.square().add(a).multiply(x).add(b);
+            ECFieldElement rhs = x.square().add(this.a).multiply(x).add(this.b);
             ECFieldElement y = rhs.sqrt();
 
             /*
@@ -486,7 +593,7 @@ public abstract class ECCurve
      */
     public static class Fp extends AbstractFp
     {
-        private static final int FP_DEFAULT_COORDS = COORD_JACOBIAN_MODIFIED;
+        private static final int FP_DEFAULT_COORDS = ECCurve.COORD_JACOBIAN_MODIFIED;
 
         BigInteger q, r;
         ECPoint.Fp infinity;
@@ -533,17 +640,17 @@ public abstract class ECCurve
 
         protected ECCurve cloneCurve()
         {
-            return new Fp(q, r, a, b, order, cofactor);
+            return new Fp(this.q, this.r, this.a, this.b, this.order, this.cofactor);
         }
 
         public boolean supportsCoordinateSystem(int coord)
         {
             switch (coord)
             {
-            case COORD_AFFINE:
-            case COORD_HOMOGENEOUS:
-            case COORD_JACOBIAN:
-            case COORD_JACOBIAN_MODIFIED:
+            case ECCurve.COORD_AFFINE:
+            case ECCurve.COORD_HOMOGENEOUS:
+            case ECCurve.COORD_JACOBIAN:
+            case ECCurve.COORD_JACOBIAN_MODIFIED:
                 return true;
             default:
                 return false;
@@ -577,13 +684,13 @@ public abstract class ECCurve
 
         public ECPoint importPoint(ECPoint p)
         {
-            if (this != p.getCurve() && this.getCoordinateSystem() == COORD_JACOBIAN && !p.isInfinity())
+            if (this != p.getCurve() && this.getCoordinateSystem() == ECCurve.COORD_JACOBIAN && !p.isInfinity())
             {
                 switch (p.getCurve().getCoordinateSystem())
                 {
-                case COORD_JACOBIAN:
-                case COORD_JACOBIAN_CHUDNOVSKY:
-                case COORD_JACOBIAN_MODIFIED:
+                case ECCurve.COORD_JACOBIAN:
+                case ECCurve.COORD_JACOBIAN_CHUDNOVSKY:
+                case ECCurve.COORD_JACOBIAN_MODIFIED:
                     return new ECPoint.Fp(this,
                         fromBigInteger(p.x.toBigInteger()),
                         fromBigInteger(p.y.toBigInteger()),
@@ -605,6 +712,18 @@ public abstract class ECCurve
 
     public static abstract class AbstractF2m extends ECCurve
     {
+        public static BigInteger inverse(int m, int[] ks, BigInteger x)
+        {
+            return new LongArray(x).modInverse(m, ks).toBigInteger();
+        }
+
+        /**
+         * The auxiliary values <code>s<sub>0</sub></code> and
+         * <code>s<sub>1</sub></code> used for partial modular reduction for
+         * Koblitz curves.
+         */
+        private BigInteger[] si = null;
+
         private static FiniteField buildField(int m, int k1, int k2, int k3)
         {
             if (k1 == 0)
@@ -639,6 +758,173 @@ public abstract class ECCurve
         {
             super(buildField(m, k1, k2, k3));
         }
+
+        public boolean isValidFieldElement(BigInteger x)
+        {
+            return x != null && x.signum() >= 0 && x.bitLength() <= this.getFieldSize();
+        }
+
+        public ECPoint createPoint(BigInteger x, BigInteger y, boolean withCompression)
+        {
+            ECFieldElement X = this.fromBigInteger(x), Y = this.fromBigInteger(y);
+
+            int coord = this.getCoordinateSystem();
+
+            switch (coord)
+            {
+            case ECCurve.COORD_LAMBDA_AFFINE:
+            case ECCurve.COORD_LAMBDA_PROJECTIVE:
+            {
+                if (X.isZero())
+                {
+                    if (!Y.square().equals(this.getB()))
+                    {
+                        throw new IllegalArgumentException();
+                    }
+                }
+                /*
+                 * NOTE: A division could be avoided using a projective result, except at present
+                 * callers will expect that the result is already normalized.
+                 */
+//                else if (coord == COORD_LAMBDA_PROJECTIVE)
+//                {
+//                    ECFieldElement Z = X;
+//                    X = X.square();
+//                    Y = Y.add(X);
+//                    return createRawPoint(X, Y, new ECFieldElement[]{ Z }, withCompression);
+//                }
+                else
+                {
+                    // Y becomes Lambda (X + Y/X) here
+                    Y = Y.divide(X).add(X);
+                }
+                break;
+            }
+            default:
+            {
+                break;
+            }
+            }
+
+            return this.createRawPoint(X, Y, withCompression);
+        }
+
+        /**
+         * Decompresses a compressed point P = (xp, yp) (X9.62 s 4.2.2).
+         * 
+         * @param yTilde
+         *            ~yp, an indication bit for the decompression of yp.
+         * @param X1
+         *            The field element xp.
+         * @return the decompressed point.
+         */
+        protected ECPoint decompressPoint(int yTilde, BigInteger X1)
+        {
+            ECFieldElement x = this.fromBigInteger(X1), y = null;
+            if (x.isZero())
+            {
+                y = this.getB().sqrt();
+            }
+            else
+            {
+                ECFieldElement beta = x.square().invert().multiply(this.getB()).add(this.getA()).add(x);
+                ECFieldElement z = solveQuadraticEquation(beta);
+                if (z != null)
+                {
+                    if (z.testBitZero() != (yTilde == 1))
+                    {
+                        z = z.addOne();
+                    }
+
+                    switch (this.getCoordinateSystem())
+                    {
+                    case ECCurve.COORD_LAMBDA_AFFINE:
+                    case ECCurve.COORD_LAMBDA_PROJECTIVE:
+                    {
+                        y = z.add(x);
+                        break;
+                    }
+                    default:
+                    {
+                        y = z.multiply(x);
+                        break;
+                    }
+                    }
+                }
+            }
+
+            if (y == null)
+            {
+                throw new IllegalArgumentException("Invalid point compression");
+            }
+
+            return this.createRawPoint(x, y, true);
+        }
+
+        /**
+         * Solves a quadratic equation <code>z<sup>2</sup> + z = beta</code>(X9.62
+         * D.1.6) The other solution is <code>z + 1</code>.
+         * 
+         * @param beta
+         *            The value to solve the quadratic equation for.
+         * @return the solution for <code>z<sup>2</sup> + z = beta</code> or
+         *         <code>null</code> if no solution exists.
+         */
+        private ECFieldElement solveQuadraticEquation(ECFieldElement beta)
+        {
+            if (beta.isZero())
+            {
+                return beta;
+            }
+
+            ECFieldElement gamma, z, zeroElement = this.fromBigInteger(ECConstants.ZERO);
+
+            int m = this.getFieldSize();
+            Random rand = new Random();
+            do
+            {
+                ECFieldElement t = this.fromBigInteger(new BigInteger(m, rand));
+                z = zeroElement;
+                ECFieldElement w = beta;
+                for (int i = 1; i < m; i++)
+                {
+                    ECFieldElement w2 = w.square();
+                    z = z.square().add(w2.multiply(t));
+                    w = w2.add(beta);
+                }
+                if (!w.isZero())
+                {
+                    return null;
+                }
+                gamma = z.square().add(z);
+            }
+            while (gamma.isZero());
+
+            return z;
+        }
+
+        /**
+         * @return the auxiliary values <code>s<sub>0</sub></code> and
+         * <code>s<sub>1</sub></code> used for partial modular reduction for
+         * Koblitz curves.
+         */
+        synchronized BigInteger[] getSi()
+        {
+            if (si == null)
+            {
+                si = Tnaf.getSi(this);
+            }
+            return si;
+        }
+
+        /**
+         * Returns true if this is a Koblitz curve (ABC curve).
+         * @return true if this is a Koblitz curve (ABC curve), false otherwise
+         */
+        public boolean isKoblitz()
+        {
+            return this.order != null && this.cofactor != null && this.b.isOne() && (this.a.isZero() || this.a.isOne());
+        }
     }
 
     /**
@@ -647,7 +933,7 @@ public abstract class ECCurve
      */
     public static class F2m extends AbstractF2m
     {
-        private static final int F2M_DEFAULT_COORDS = COORD_LAMBDA_PROJECTIVE;
+        private static final int F2M_DEFAULT_COORDS = ECCurve.COORD_LAMBDA_PROJECTIVE;
 
         /**
          * The exponent <code>m</code> of <code>F<sub>2<sup>m</sup></sub></code>.
@@ -685,19 +971,6 @@ public abstract class ECCurve
          */
         private ECPoint.F2m infinity;  // can't be final - JDK 1.1
 
-        /**
-         * The parameter <code>&mu;</code> of the elliptic curve if this is
-         * a Koblitz curve.
-         */
-        private byte mu = 0;
-
-        /**
-         * The auxiliary values <code>s<sub>0</sub></code> and
-         * <code>s<sub>1</sub></code> used for partial modular reduction for
-         * Koblitz curves.
-         */
-        private BigInteger[] si = null;
-
         /**
          * Constructor for Trinomial Polynomial Basis (TPB).
          * @param m  The exponent <code>m</code> of
@@ -847,16 +1120,16 @@ public abstract class ECCurve
 
         protected ECCurve cloneCurve()
         {
-            return new F2m(m, k1, k2, k3, a, b, order, cofactor);
+            return new F2m(this.m, this.k1, this.k2, this.k3, this.a, this.b, this.order, this.cofactor);
         }
 
         public boolean supportsCoordinateSystem(int coord)
         {
             switch (coord)
             {
-            case COORD_AFFINE:
-            case COORD_HOMOGENEOUS:
-            case COORD_LAMBDA_PROJECTIVE:
+            case ECCurve.COORD_AFFINE:
+            case ECCurve.COORD_HOMOGENEOUS:
+            case ECCurve.COORD_LAMBDA_PROJECTIVE:
                 return true;
             default:
                 return false;
@@ -883,38 +1156,6 @@ public abstract class ECCurve
             return new ECFieldElement.F2m(this.m, this.k1, this.k2, this.k3, x);
         }
 
-        public ECPoint createPoint(BigInteger x, BigInteger y, boolean withCompression)
-        {
-            ECFieldElement X = fromBigInteger(x), Y = fromBigInteger(y);
-
-            switch (this.getCoordinateSystem())
-            {
-            case COORD_LAMBDA_AFFINE:
-            case COORD_LAMBDA_PROJECTIVE:
-            {
-                if (X.isZero())
-                {
-                    if (!Y.square().equals(this.getB()))
-                    {
-                        throw new IllegalArgumentException();
-                    }
-                }
-                else
-                {
-                    // Y becomes Lambda (X + Y/X) here
-                    Y = Y.divide(X).add(X);
-                }
-                break;
-            }
-            default:
-            {
-                break;
-            }
-            }
-
-            return createRawPoint(X, Y, withCompression);
-        }
-
         protected ECPoint createRawPoint(ECFieldElement x, ECFieldElement y, boolean withCompression)
         {
             return new ECPoint.F2m(this, x, y, withCompression);
@@ -930,140 +1171,6 @@ public abstract class ECCurve
             return infinity;
         }
 
-        /**
-         * Returns true if this is a Koblitz curve (ABC curve).
-         * @return true if this is a Koblitz curve (ABC curve), false otherwise
-         */
-        public boolean isKoblitz()
-        {
-            return order != null && cofactor != null && b.isOne() && (a.isZero() || a.isOne());
-        }
-
-        /**
-         * Returns the parameter <code>&mu;</code> of the elliptic curve.
-         * @return <code>&mu;</code> of the elliptic curve.
-         * @throws IllegalArgumentException if the given ECCurve is not a
-         * Koblitz curve.
-         */
-        synchronized byte getMu()
-        {
-            if (mu == 0)
-            {
-                mu = Tnaf.getMu(this);
-            }
-            return mu;
-        }
-
-        /**
-         * @return the auxiliary values <code>s<sub>0</sub></code> and
-         * <code>s<sub>1</sub></code> used for partial modular reduction for
-         * Koblitz curves.
-         */
-        synchronized BigInteger[] getSi()
-        {
-            if (si == null)
-            {
-                si = Tnaf.getSi(this);
-            }
-            return si;
-        }
-
-        /**
-         * Decompresses a compressed point P = (xp, yp) (X9.62 s 4.2.2).
-         * 
-         * @param yTilde
-         *            ~yp, an indication bit for the decompression of yp.
-         * @param X1
-         *            The field element xp.
-         * @return the decompressed point.
-         */
-        protected ECPoint decompressPoint(int yTilde, BigInteger X1)
-        {
-            ECFieldElement x = fromBigInteger(X1), y = null;
-            if (x.isZero())
-            {
-                y = b.sqrt();
-            }
-            else
-            {
-                ECFieldElement beta = x.square().invert().multiply(b).add(a).add(x);
-                ECFieldElement z = solveQuadraticEquation(beta);
-                if (z != null)
-                {
-                    if (z.testBitZero() != (yTilde == 1))
-                    {
-                        z = z.addOne();
-                    }
-
-                    switch (this.getCoordinateSystem())
-                    {
-                    case COORD_LAMBDA_AFFINE:
-                    case COORD_LAMBDA_PROJECTIVE:
-                    {
-                        y = z.add(x);
-                        break;
-                    }
-                    default:
-                    {
-                        y = z.multiply(x);
-                        break;
-                    }
-                    }
-                }
-            }
-
-            if (y == null)
-            {
-                throw new IllegalArgumentException("Invalid point compression");
-            }
-
-            return this.createRawPoint(x, y, true);
-        }
-
-        /**
-         * Solves a quadratic equation <code>z<sup>2</sup> + z = beta</code>(X9.62
-         * D.1.6) The other solution is <code>z + 1</code>.
-         * 
-         * @param beta
-         *            The value to solve the quadratic equation for.
-         * @return the solution for <code>z<sup>2</sup> + z = beta</code> or
-         *         <code>null</code> if no solution exists.
-         */
-        private ECFieldElement solveQuadraticEquation(ECFieldElement beta)
-        {
-            if (beta.isZero())
-            {
-                return beta;
-            }
-
-            ECFieldElement zeroElement = fromBigInteger(ECConstants.ZERO);
-
-            ECFieldElement z = null;
-            ECFieldElement gamma = null;
-
-            Random rand = new Random();
-            do
-            {
-                ECFieldElement t = fromBigInteger(new BigInteger(m, rand));
-                z = zeroElement;
-                ECFieldElement w = beta;
-                for (int i = 1; i <= m - 1; i++)
-                {
-                    ECFieldElement w2 = w.square();
-                    z = z.square().add(w2.multiply(t));
-                    w = w2.add(beta);
-                }
-                if (!w.isZero())
-                {
-                    return null;
-                }
-                gamma = z.square().add(z);
-            }
-            while (gamma.isZero());
-
-            return z;
-        }
-
         public int getM()
         {
             return m;
@@ -1099,7 +1206,7 @@ public abstract class ECCurve
          */
         public BigInteger getN()
         {
-            return order;
+            return this.order;
         }
 
         /**
@@ -1107,7 +1214,53 @@ public abstract class ECCurve
          */
         public BigInteger getH()
         {
-            return cofactor;
+            return this.cofactor;
+        }
+
+        public ECLookupTable createCacheSafeLookupTable(ECPoint[] points, int off, final int len)
+        {
+            final int FE_LONGS = (m + 63) >>> 6;
+            final int[] ks = isTrinomial() ? new int[]{ k1 } : new int[]{ k1, k2, k3 }; 
+
+            final long[] table = new long[len * FE_LONGS * 2];
+            {
+                int pos = 0;
+                for (int i = 0; i < len; ++i)
+                {
+                    ECPoint p = points[off + i];
+                    ((ECFieldElement.F2m)p.getRawXCoord()).x.copyTo(table, pos); pos += FE_LONGS;
+                    ((ECFieldElement.F2m)p.getRawYCoord()).x.copyTo(table, pos); pos += FE_LONGS;
+                }
+            }
+
+            return new ECLookupTable()
+            {
+                public int getSize()
+                {
+                    return len;
+                }
+
+                public ECPoint lookup(int index)
+                {
+                    long[] x = Nat.create64(FE_LONGS), y = Nat.create64(FE_LONGS);
+                    int pos = 0;
+
+                    for (int i = 0; i < len; ++i)
+                    {
+                        long MASK = ((i ^ index) - 1) >> 31;
+
+                        for (int j = 0; j < FE_LONGS; ++j)
+                        {
+                            x[j] ^= table[pos + j] & MASK;
+                            y[j] ^= table[pos + FE_LONGS + j] & MASK;
+                        }
+
+                        pos += (FE_LONGS * 2);
+                    }
+
+                    return createRawPoint(new ECFieldElement.F2m(m, ks, new LongArray(x)), new ECFieldElement.F2m(m, ks, new LongArray(y)), false);
+                }
+            };
         }
     }
 }
