@@ -59,6 +59,16 @@ public abstract class ECFieldElement
         return square().add(x.multiply(y));
     }
 
+    public ECFieldElement squarePow(int pow)
+    {
+        ECFieldElement r = this;
+        for (int i = 0; i < pow; ++i)
+        {
+            r = r.square();
+        }
+        return r;
+    }
+
     public boolean testBitZero()
     {
         return toBigInteger().testBit(0);
@@ -523,7 +533,7 @@ public abstract class ECFieldElement
         /**
          * The <code>LongArray</code> holding the bits.
          */
-        private LongArray x;
+        LongArray x;
 
         /**
          * Constructor for PPB.
@@ -548,6 +558,11 @@ public abstract class ECFieldElement
             int k3,
             BigInteger x)
         {
+            if (x == null || x.signum() < 0 || x.bitLength() > m)
+            {
+                throw new IllegalArgumentException("x value invalid in F2m field element");
+            }
+
             if ((k2 == 0) && (k3 == 0))
             {
                 this.representation = TPB;
@@ -589,7 +604,7 @@ public abstract class ECFieldElement
             this(m, k, 0, 0, x);
         }
 
-        private F2m(int m, int[] ks, LongArray x)
+        F2m(int m, int[] ks, LongArray x)
         {
             this.m = m;
             this.representation = (ks.length == 1) ? TPB : PPB;
@@ -766,6 +781,11 @@ public abstract class ECFieldElement
             return new F2m(m, ks, aa);
         }
 
+        public ECFieldElement squarePow(int pow)
+        {
+            return pow < 1 ? this : new F2m(m, ks, x.modSquareN(pow, m, ks));
+        }
+
         public ECFieldElement invert()
         {
             return new ECFieldElement.F2m(this.m, this.ks, this.x.modInverse(m, ks));
@@ -773,14 +793,7 @@ public abstract class ECFieldElement
 
         public ECFieldElement sqrt()
         {
-            LongArray x1 = this.x;
-            if (x1.isOne() || x1.isZero())
-            {
-                return this;
-            }
-
-            LongArray x2 = x1.modSquareN(m - 1, m, ks);
-            return new ECFieldElement.F2m(m, ks, x2);
+            return (x.isZero() || x.isOne()) ? this : squarePow(m - 1);
         }
 
         /**
