@@ -110,12 +110,26 @@ public abstract class Nat128
         z[3] = x[3];
     }
 
+    public static void copy(int[] x, int xOff, int[] z, int zOff)
+    {
+        z[zOff + 0] = x[xOff + 0];
+        z[zOff + 1] = x[xOff + 1];
+        z[zOff + 2] = x[xOff + 2];
+        z[zOff + 3] = x[xOff + 3];
+    }
+
     public static void copy64(long[] x, long[] z)
     {
         z[0] = x[0];
         z[1] = x[1];
     }
 
+    public static void copy64(long[] x, int xOff, long[] z, int zOff)
+    {
+        z[zOff + 0] = x[xOff + 0];
+        z[zOff + 1] = x[xOff + 1];
+    }
+
     public static int[] create()
     {
         return new int[4];
