@@ -8,7 +8,7 @@ import java.util.NoSuchElementException;
  */
 public final class Arrays
 {
-    private Arrays() 
+    private Arrays()
     {
         // static class, hide constructor
     }
@@ -135,36 +135,37 @@ public final class Arrays
 
     /**
      * A constant time equals comparison - does not terminate early if
-     * test will fail.
+     * test will fail. For best results always pass the expected value
+     * as the first parameter.
      *
-     * @param a first array
-     * @param b second array
+     * @param expected first array
+     * @param supplied second array
      * @return true if arrays equal, false otherwise.
      */
     public static boolean constantTimeAreEqual(
-        byte[]  a,
-        byte[]  b)
+        byte[]  expected,
+        byte[]  supplied)
     {
-        if (a == b)
+        if (expected == supplied)
         {
             return true;
         }
 
-        if (a == null || b == null)
+        if (expected == null || supplied == null)
         {
             return false;
         }
 
-        if (a.length != b.length)
+        if (expected.length != supplied.length)
         {
-            return false;
+            return !Arrays.constantTimeAreEqual(expected, expected);
         }
 
         int nonEqual = 0;
 
-        for (int i = 0; i != a.length; i++)
+        for (int i = 0; i != expected.length; i++)
         {
-            nonEqual |= (a[i] ^ b[i]);
+            nonEqual |= (expected[i] ^ supplied[i]);
         }
 
         return nonEqual == 0;
@@ -355,7 +356,7 @@ public final class Arrays
     }
 
     public static void fill(
-        short[] array, 
+        short[] array,
         short value)
     {
         for (int i = 0; i < array.length; i++)
@@ -373,7 +374,63 @@ public final class Arrays
             array[i] = value;
         }
     }
-    
+
+    public static void fill(
+        byte[] array,
+        int out,
+        byte value)
+    {
+        if(out < array.length)
+        {
+            for (int i = out; i < array.length; i++)
+            {
+                array[i] = value;
+            }
+        }
+    }
+
+    public static void fill(
+        int[] array,
+        int out,
+        int value)
+    {
+        if(out < array.length)
+        {
+            for (int i = out; i < array.length; i++)
+            {
+                array[i] = value;
+            }
+        }
+    }
+
+    public static void fill(
+        short[] array,
+        int out,
+        short value)
+    {
+        if(out < array.length)
+        {
+            for (int i = out; i < array.length; i++)
+            {
+                array[i] = value;
+            }
+        }
+    }
+
+    public static void fill(
+        long[] array,
+        int out,
+        long value)
+    {
+        if(out < array.length)
+        {
+            for (int i = out; i < array.length; i++)
+            {
+                array[i] = value;
+            }
+        }
+    }
+
     public static int hashCode(byte[] data)
     {
         if (data == null)
@@ -681,9 +738,9 @@ public final class Arrays
             return null;
         }
         long[] copy = new long[data.length];
-        
+
         System.arraycopy(data, 0, copy, 0, data.length);
-        
+
         return copy;
     }
 
@@ -944,6 +1001,22 @@ public final class Arrays
         return result;
     }
 
+    public static String[] append(String[] a, String b)
+    {
+        if (a == null)
+        {
+            return new String[]{ b };
+        }
+
+        int length = a.length;
+        String[] result = new String[length + 1];
+        System.arraycopy(a, 0, result, 0, length);
+        result[length] = b;
+        return result;
+    }
+
+
+
     public static byte[] concatenate(byte[] a, byte[] b)
     {
         if (a != null && b != null)
@@ -1110,7 +1183,7 @@ public final class Arrays
 
         int p1 = 0, p2 = a.length;
         byte[] result = new byte[p2];
-        
+
         while (--p2 >= 0)
         {
             result[p2] = a[p1++];
@@ -1180,4 +1253,20 @@ public final class Arrays
             throw new UnsupportedOperationException("Cannot remove element from an Array.");
         }
     }
+
+    /**
+     * Fill input array by zeros
+     *
+     * @param array input array
+     */
+    public static void clear(byte[] array)
+    {
+        if (array != null)
+        {
+            for (int i = 0; i < array.length; i++)
+            {
+                array[i] = 0;
+            }
+        }
+    }
 }
