@@ -159,7 +159,8 @@ public class X500NameTest
         // compatibility encoding
         testEncodingGeneralizedTime(BCStyle.DATE_OF_BIRTH, "20020122122220Z");
         testEncodingUTF8String(BCStyle.CN, "Mörsky");
-
+        testEncodingUTF8String(BCStyle.ORGANIZATION_IDENTIFIER, "Mörsky");
+        
         //
         // composite
         //
@@ -262,7 +263,7 @@ public class X500NameTest
         val1.addElement("feedback-crypto@bouncycastle.org");
 
         name1 = new X500Name(ord1, val1);
-        
+
         Vector values = name1.getValues();
         if (!compareVectors(values, val1))
         {
@@ -566,7 +567,7 @@ public class X500NameTest
         {
             fail("failed X509DefaultEntryConverter test got " + converted + " expected: " + hexEncodedString);
         }
-        
+
         //
         // try a weird value
         //
@@ -668,6 +669,12 @@ public class X500NameTest
         {
             fail("unknown oid matched aliases!");
         }
+
+        if (!new X500Name("CN=\"  CA1 -   CP.04.03\", OU=Testing, OU=Dod, O=U.S. Government, C=US")
+            .equals(new X500Name("CN=\"ca1 - CP.04.03  \", OU=Testing, OU=Dod, O=U.S. Government, C=US")))
+        {
+            fail("padded equality test failed");
+        }
     }
 
     private String getValue(RDN vl)
