@@ -7,15 +7,19 @@ import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
 import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.Signer;
 import org.bouncycastle.crypto.digests.SHA224Digest;
+import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
 import org.bouncycastle.crypto.params.ParametersWithRandom;
 import org.bouncycastle.pqc.crypto.DigestingMessageSigner;
+import org.bouncycastle.pqc.crypto.DigestingStateAwareMessageSigner;
 import org.bouncycastle.pqc.crypto.gmss.GMSSDigestProvider;
 import org.bouncycastle.pqc.crypto.gmss.GMSSKeyGenerationParameters;
 import org.bouncycastle.pqc.crypto.gmss.GMSSKeyPairGenerator;
 import org.bouncycastle.pqc.crypto.gmss.GMSSParameters;
 import org.bouncycastle.pqc.crypto.gmss.GMSSPrivateKeyParameters;
 import org.bouncycastle.pqc.crypto.gmss.GMSSSigner;
+import org.bouncycastle.pqc.crypto.gmss.GMSSStateAwareSigner;
 import org.bouncycastle.util.BigIntegers;
+import org.bouncycastle.util.Strings;
 import org.bouncycastle.util.encoders.Hex;
 import org.bouncycastle.util.test.FixedSecureRandom;
 import org.bouncycastle.util.test.SimpleTest;
@@ -27,7 +31,7 @@ public class GMSSSignerTest
     byte[] keyData = Hex.decode("b5014e4b60ef2ba8b6211b4062ba3224e0427dd3");
 
     SecureRandom keyRandom = new FixedSecureRandom(
-        new FixedSecureRandom.Source[] { new FixedSecureRandom.Data(keyData), new FixedSecureRandom.Data(keyData) });
+        new FixedSecureRandom.Source[]{new FixedSecureRandom.Data(keyData), new FixedSecureRandom.Data(keyData)});
 
     public String getName()
     {
@@ -57,7 +61,9 @@ public class GMSSSignerTest
 
         AsymmetricCipherKeyPair pair = gmssKeyGen.generateKeyPair();
 
-        ParametersWithRandom param = new ParametersWithRandom(pair.getPrivate(), keyRandom);
+        GMSSPrivateKeyParameters privKey = (GMSSPrivateKeyParameters)pair.getPrivate();
+
+        ParametersWithRandom param = new ParametersWithRandom(privKey, keyRandom);
 
         // TODO
         Signer gmssSigner = new DigestingMessageSigner(new GMSSSigner(digProvider), new SHA224Digest());
@@ -79,6 +85,50 @@ public class GMSSSignerTest
         {
             fail("private key not marked as used");
         }
+
+        stateAwareTest(privKey.nextKey(), pair.getPublic());
+    }
+
+    private void stateAwareTest(GMSSPrivateKeyParameters privKey, AsymmetricKeyParameter pub)
+    {
+        DigestingStateAwareMessageSigner statefulSigner = new DigestingStateAwareMessageSigner(new GMSSStateAwareSigner(new SHA224Digest()), new SHA224Digest());
+        statefulSigner.init(true, new ParametersWithRandom(privKey, new SecureRandom()));
+
+        byte[] mes1 = Strings.toByteArray("Message One");
+        statefulSigner.update(mes1, 0, mes1.length);
+        byte[] sig1 = statefulSigner.generateSignature();
+
+        isTrue(privKey.isUsed());
+
+        byte[] mes2 = Strings.toByteArray("Message Two");
+        statefulSigner.update(mes2, 0, mes2.length);
+        byte[] sig2 = statefulSigner.generateSignature();
+
+        GMSSPrivateKeyParameters recoveredKey = (GMSSPrivateKeyParameters)statefulSigner.getUpdatedPrivateKey();
+
+        isTrue(recoveredKey.isUsed() == false);
+
+        try
+        {
+            statefulSigner.generateSignature();
+        }
+        catch (IllegalStateException e)
+        {
+            isEquals("signing key no longer usable", e.getMessage());
+        }
+
+        statefulSigner.init(false, pub);
+        statefulSigner.update(mes2, 0, mes2.length);
+        if (!statefulSigner.verifySignature(sig2))
+        {
+            fail("verification two fails");
+        }
+
+        statefulSigner.update(mes1, 0, mes1.length);
+        if (!statefulSigner.verifySignature(sig1))
+        {
+            fail("verification one fails");
+        }
     }
 
     public static void main(
