@@ -8,6 +8,7 @@ import java.security.KeyPair;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.PrivateKey;
+import java.security.Security;
 import java.security.cert.CertificateEncodingException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
@@ -25,15 +26,18 @@ import junit.framework.TestSuite;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.cms.CMSAlgorithm;
+import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
 import org.bouncycastle.cms.KeyTransRecipientId;
 import org.bouncycastle.cms.RecipientId;
 import org.bouncycastle.cms.RecipientInformation;
 import org.bouncycastle.cms.RecipientInformationStore;
 import org.bouncycastle.cms.jcajce.JceCMSContentEncryptorBuilder;
+import org.bouncycastle.cms.jcajce.JceKeyAgreeEnvelopedRecipient;
+import org.bouncycastle.cms.jcajce.JceKeyAgreeRecipientId;
+import org.bouncycastle.cms.jcajce.JceKeyAgreeRecipientInfoGenerator;
 import org.bouncycastle.cms.jcajce.JceKeyTransEnvelopedRecipient;
 import org.bouncycastle.cms.jcajce.JceKeyTransRecipientId;
 import org.bouncycastle.cms.jcajce.JceKeyTransRecipientInfoGenerator;
-import org.bouncycastle.cms.test.CMSTestUtil;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.mail.smime.SMIMEEnveloped;
 import org.bouncycastle.mail.smime.SMIMEEnvelopedGenerator;
@@ -61,6 +65,12 @@ public class NewSMIMEEnvelopedTest
     private static KeyPair         _reciKP2;
     private static X509Certificate _reciCert2;
 
+    private static KeyPair         _origEcKP;
+    private static KeyPair         _reciEcKP;
+    private static X509Certificate _reciEcCert;
+    private static KeyPair         _reciEcKP2;
+    private static X509Certificate _reciEcCert2;
+
     private static boolean         _initialised = false;
 
     private static final byte[] testMessage = Base64.decode(
@@ -78,6 +88,11 @@ public class NewSMIMEEnvelopedTest
     {
         if (!_initialised)
         {
+            if (Security.getProvider("BC") == null)
+            {
+                Security.addProvider(new BouncyCastleProvider());
+            }
+
             _initialised = true;
 
             _signDN   = "O=Bouncy Castle, C=AU";
@@ -90,6 +105,12 @@ public class NewSMIMEEnvelopedTest
             _reciDN2   = "CN=Fred, OU=Sales, O=Bouncy Castle, C=AU";
             _reciKP2   = CMSTestUtil.makeKeyPair();
             _reciCert2 = CMSTestUtil.makeCertificate(_reciKP2, _reciDN2, _signKP, _signDN);
+
+            _origEcKP = CMSTestUtil.makeEcDsaKeyPair();
+            _reciEcKP = CMSTestUtil.makeEcDsaKeyPair();
+            _reciEcCert = CMSTestUtil.makeCertificate(_reciEcKP, _reciDN, _signKP, _signDN);
+            _reciEcKP2 = CMSTestUtil.makeEcDsaKeyPair();
+            _reciEcCert2 = CMSTestUtil.makeCertificate(_reciEcKP2, _reciDN2, _signKP, _signDN);
         }
     }
 
@@ -283,7 +304,7 @@ public class NewSMIMEEnvelopedTest
 
         MimeBodyPart    res = SMIMEUtil.toMimeBodyPart(recipient.getContent(new JceKeyTransEnvelopedRecipient(_reciKP.getPrivate()).setProvider(BC)));
 
-        verifyMessageBytes(msg, res);
+        SMIMETestUtil.verifyMessageBytes(msg, res);
     }
 
     public void testDotNetEncMailMatch()
@@ -386,7 +407,7 @@ public class NewSMIMEEnvelopedTest
 
         MimeBodyPart    res = SMIMEUtil.toMimeBodyPart(recipient.getContent(new JceKeyTransEnvelopedRecipient(_reciKP.getPrivate()).setProvider(BC)));
 
-        verifyMessageBytes(msg, res);
+        SMIMETestUtil.verifyMessageBytes(msg, res);
     }
     
     public void testTwoRecipients()
@@ -414,7 +435,7 @@ public class NewSMIMEEnvelopedTest
         
         FileBackedMimeBodyPart    res = SMIMEUtil.toMimeBodyPart(recipient.getContentStream(new JceKeyTransEnvelopedRecipient(_reciKP2.getPrivate()).setProvider(BC)));
 
-        verifyMessageBytes(_msg, res);
+        SMIMETestUtil.verifyMessageBytes(_msg, res);
         
         m = new SMIMEEnvelopedParser(mp);
 
@@ -427,7 +448,7 @@ public class NewSMIMEEnvelopedTest
  
         res = SMIMEUtil.toMimeBodyPart(recipient.getContentStream(new JceKeyTransEnvelopedRecipient(_reciKP.getPrivate()).setProvider(BC)));
 
-        verifyMessageBytes(_msg, res);
+        SMIMETestUtil.verifyMessageBytes(_msg, res);
         
         res.dispose();
     }
@@ -455,7 +476,7 @@ public class NewSMIMEEnvelopedTest
 
         MimeBodyPart    res = SMIMEUtil.toMimeBodyPart(recipient.getContent(new JceKeyTransEnvelopedRecipient(_reciKP.getPrivate()).setProvider(BC)));
 
-        verifyMessageBytes(msg, res);
+        SMIMETestUtil.verifyMessageBytes(msg, res);
     }
     
     private void verifyParserAlgorithm(
@@ -481,7 +502,7 @@ public class NewSMIMEEnvelopedTest
 
         MimeBodyPart    res = SMIMEUtil.toMimeBodyPart(recipient.getContent(new JceKeyTransEnvelopedRecipient(_reciKP.getPrivate()).setProvider(BC)));
 
-        verifyMessageBytes(msg, res);
+        SMIMETestUtil.verifyMessageBytes(msg, res);
     }
 
     private RecipientId getRecipientId(
@@ -492,20 +513,73 @@ public class NewSMIMEEnvelopedTest
 
         return recId;
     }
-    
-    
-    private void verifyMessageBytes(MimeBodyPart a, MimeBodyPart b) 
-        throws IOException, MessagingException
+
+    public void testKDFAgreements()
+            throws Exception
     {
-        ByteArrayOutputStream _baos = new ByteArrayOutputStream();
-        a.writeTo(_baos);
-        _baos.close();
-        byte[] _msgBytes = _baos.toByteArray();
-        _baos = new ByteArrayOutputStream();
-        b.writeTo(_baos);
-        _baos.close();
-        byte[] _resBytes = _baos.toByteArray();
-        
-        assertEquals(true, Arrays.equals(_msgBytes, _resBytes));
+        MimeBodyPart    msg      = SMIMETestUtil.makeMimeBodyPart("WallaWallaWashington");
+
+        doTryAgreement(msg, CMSAlgorithm.ECDH_SHA1KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECDH_SHA224KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECDH_SHA256KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECDH_SHA384KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECDH_SHA512KDF);
+
+        doTryAgreement(msg, CMSAlgorithm.ECCDH_SHA1KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECCDH_SHA224KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECCDH_SHA256KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECCDH_SHA384KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECCDH_SHA512KDF);
+
+        doTryAgreement(msg, CMSAlgorithm.ECMQV_SHA1KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECMQV_SHA224KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECMQV_SHA256KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECMQV_SHA384KDF);
+        doTryAgreement(msg, CMSAlgorithm.ECMQV_SHA512KDF);
+    }
+
+    private void doTryAgreement(MimeBodyPart data, ASN1ObjectIdentifier algorithm)
+        throws Exception
+    {
+        SMIMEEnvelopedGenerator edGen = new SMIMEEnvelopedGenerator();
+
+        edGen.addRecipientInfoGenerator(new JceKeyAgreeRecipientInfoGenerator(algorithm,
+            _origEcKP.getPrivate(), _origEcKP.getPublic(),
+            CMSAlgorithm.AES128_WRAP).addRecipient(_reciEcCert).setProvider(BC));
+
+        MimeBodyPart res = edGen.generate(
+            data,
+            new JceCMSContentEncryptorBuilder(CMSAlgorithm.AES128_CBC).setProvider(BC).build());
+
+        SMIMEEnveloped ed = new SMIMEEnveloped(res);
+
+        assertEquals(ed.getEncryptionAlgOID(), CMSEnvelopedDataGenerator.AES128_CBC);
+
+        RecipientInformationStore recipients = ed.getRecipientInfos();
+
+        confirmDataReceived(recipients, data, _reciEcCert, _reciEcKP.getPrivate(), BC);
+        confirmNumberRecipients(recipients, 1);
+    }
+
+    private static void confirmDataReceived(RecipientInformationStore recipients,
+       MimeBodyPart  expectedData, X509Certificate reciCert, PrivateKey reciPrivKey, String provider)
+        throws Exception
+    {
+        RecipientId rid = new JceKeyAgreeRecipientId(reciCert);
+
+        RecipientInformation recipient = recipients.get(rid);
+        assertNotNull(recipient);
+
+        ByteArrayOutputStream bOut = new ByteArrayOutputStream();
+
+        expectedData.writeTo(bOut);
+
+        byte[] actualData = recipient.getContent(new JceKeyAgreeEnvelopedRecipient(reciPrivKey).setProvider(provider));
+        assertEquals(true, Arrays.equals(bOut.toByteArray(), actualData));
+    }
+
+    private static void confirmNumberRecipients(RecipientInformationStore recipients, int count)
+    {
+        assertEquals(count, recipients.getRecipients().size());
     }
 }
