@@ -23,7 +23,9 @@ import javax.crypto.KeyGenerator;
 import javax.crypto.Mac;
 import javax.crypto.NoSuchPaddingException;
 import javax.crypto.SecretKey;
+import javax.crypto.SecretKeyFactory;
 import javax.crypto.spec.IvParameterSpec;
+import javax.crypto.spec.PBEKeySpec;
 import javax.crypto.spec.RC2ParameterSpec;
 import javax.crypto.spec.SecretKeySpec;
 
@@ -33,17 +35,21 @@ import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.DERNull;
 import org.bouncycastle.asn1.DEROctetString;
+import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
+import org.bouncycastle.asn1.pkcs.PBKDF2Params;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.RC2CBCParameter;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.cms.CMSAlgorithm;
 import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
 import org.bouncycastle.cms.CMSException;
+import org.bouncycastle.cms.PasswordRecipient;
 import org.bouncycastle.operator.DefaultSecretKeySizeProvider;
 import org.bouncycastle.operator.GenericKey;
 import org.bouncycastle.operator.SecretKeySizeProvider;
 import org.bouncycastle.operator.SymmetricKeyUnwrapper;
 import org.bouncycastle.operator.jcajce.JceAsymmetricKeyUnwrapper;
+import org.bouncycastle.operator.jcajce.JceKTSKeyUnwrapper;
 
 public class EnvelopedDataHelper
 {
@@ -53,6 +59,8 @@ public class EnvelopedDataHelper
     protected static final Map CIPHER_ALG_NAMES = new HashMap();
     protected static final Map MAC_ALG_NAMES = new HashMap();
 
+    private static final Map PBKDF2_ALG_NAMES = new HashMap();
+
     static
     {
         BASE_CIPHER_NAMES.put(CMSAlgorithm.DES_CBC,  "DES");
@@ -67,6 +75,7 @@ public class EnvelopedDataHelper
         BASE_CIPHER_NAMES.put(CMSAlgorithm.CAMELLIA256_CBC, "Camellia");
         BASE_CIPHER_NAMES.put(CMSAlgorithm.SEED_CBC, "SEED");
         BASE_CIPHER_NAMES.put(PKCSObjectIdentifiers.rc4, "RC4");
+        BASE_CIPHER_NAMES.put(CryptoProObjectIdentifiers.gostR28147_gcfb, "GOST28147");
 
         CIPHER_ALG_NAMES.put(CMSAlgorithm.DES_CBC,  "DES/CBC/PKCS5Padding");
         CIPHER_ALG_NAMES.put(CMSAlgorithm.RC2_CBC,  "RC2/CBC/PKCS5Padding");
@@ -87,6 +96,12 @@ public class EnvelopedDataHelper
         MAC_ALG_NAMES.put(CMSAlgorithm.AES192_CBC,  "AESMac");
         MAC_ALG_NAMES.put(CMSAlgorithm.AES256_CBC,  "AESMac");
         MAC_ALG_NAMES.put(CMSAlgorithm.RC2_CBC,  "RC2Mac");
+
+        PBKDF2_ALG_NAMES.put(PasswordRecipient.PRF.HMacSHA1.getAlgorithmID(), "PBKDF2WITHHMACSHA1");
+        PBKDF2_ALG_NAMES.put(PasswordRecipient.PRF.HMacSHA224.getAlgorithmID(), "PBKDF2WITHHMACSHA224");
+        PBKDF2_ALG_NAMES.put(PasswordRecipient.PRF.HMacSHA256.getAlgorithmID(), "PBKDF2WITHHMACSHA256");
+        PBKDF2_ALG_NAMES.put(PasswordRecipient.PRF.HMacSHA384.getAlgorithmID(), "PBKDF2WITHHMACSHA384");
+        PBKDF2_ALG_NAMES.put(PasswordRecipient.PRF.HMacSHA512.getAlgorithmID(), "PBKDF2WITHHMACSHA512");
     }
 
     private static final short[] rc2Table = {
@@ -302,7 +317,7 @@ public class EnvelopedDataHelper
         }
         catch (GeneralSecurityException e)
         {
-            throw new CMSException("cannot create key pair generator: " + e.getMessage(), e);
+            throw new CMSException("cannot create key agreement: " + e.getMessage(), e);
         }
     }
 
@@ -410,7 +425,7 @@ public class EnvelopedDataHelper
 
                         CMSUtils.loadParameters(params, sParams);
 
-                        mac.init(sKey, params.getParameterSpec(IvParameterSpec.class));
+                        mac.init(sKey, params.getParameterSpec(AlgorithmParameterSpec.class));
                     }
                     catch (NoSuchAlgorithmException e)
                     {
@@ -619,6 +634,11 @@ public class EnvelopedDataHelper
         return helper.createAsymmetricUnwrapper(keyEncryptionAlgorithm, keyEncryptionKey);
     }
 
+    public JceKTSKeyUnwrapper createAsymmetricUnwrapper(AlgorithmIdentifier keyEncryptionAlgorithm, PrivateKey keyEncryptionKey, byte[] partyUInfo, byte[] partyVInfo)
+    {
+        return helper.createAsymmetricUnwrapper(keyEncryptionAlgorithm, keyEncryptionKey, partyUInfo, partyVInfo);
+    }
+
     public SymmetricKeyUnwrapper createSymmetricUnwrapper(AlgorithmIdentifier keyEncryptionAlgorithm, SecretKey keyEncryptionKey)
     {
         return helper.createSymmetricUnwrapper(keyEncryptionAlgorithm, keyEncryptionKey);
@@ -659,6 +679,40 @@ public class EnvelopedDataHelper
         throw new IllegalStateException("unknown parameter spec: " + paramSpec);
     }
 
+    SecretKeyFactory createSecretKeyFactory(String keyFactoryAlgorithm)
+        throws NoSuchProviderException, NoSuchAlgorithmException
+    {
+        return helper.createSecretKeyFactory(keyFactoryAlgorithm);
+    }
+
+    byte[] calculateDerivedKey(int schemeID, char[] password, AlgorithmIdentifier derivationAlgorithm, int keySize)
+        throws CMSException
+    {
+        PBKDF2Params params = PBKDF2Params.getInstance(derivationAlgorithm.getParameters());
+
+        try
+        {
+            SecretKeyFactory keyFact;
+
+            if (schemeID == PasswordRecipient.PKCS5_SCHEME2)
+            {
+                keyFact = helper.createSecretKeyFactory("PBKDF2with8BIT");
+            }
+            else
+            {
+                keyFact = helper.createSecretKeyFactory((String)PBKDF2_ALG_NAMES.get(params.getPrf()));
+            }
+
+            SecretKey key = keyFact.generateSecret(new PBEKeySpec(password, params.getSalt(), params.getIterationCount().intValue(), keySize));
+
+            return key.getEncoded();
+        }
+        catch (GeneralSecurityException e)
+        {
+             throw new CMSException("Unable to calculate derived key from password: " + e.getMessage(), e);
+        }
+    }
+
     static interface JCECallback
     {
         Object doInJCE()
