@@ -3,13 +3,17 @@ package org.bouncycastle.tsp;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.math.BigInteger;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Date;
+import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
+import java.util.SimpleTimeZone;
 
 import org.bouncycastle.asn1.ASN1Boolean;
 import org.bouncycastle.asn1.ASN1Encoding;
@@ -28,6 +32,8 @@ import org.bouncycastle.asn1.tsp.Accuracy;
 import org.bouncycastle.asn1.tsp.MessageImprint;
 import org.bouncycastle.asn1.tsp.TSTInfo;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
+import org.bouncycastle.asn1.x509.Extensions;
+import org.bouncycastle.asn1.x509.ExtensionsGenerator;
 import org.bouncycastle.asn1.x509.GeneralName;
 import org.bouncycastle.asn1.x509.GeneralNames;
 import org.bouncycastle.asn1.x509.IssuerSerial;
@@ -71,11 +77,34 @@ import org.bouncycastle.util.Store;
  */
 public class TimeStampTokenGenerator
 {
-    int accuracySeconds = -1;
+    /**
+     * Create time-stamps with a resolution of 1 second (the default).
+     */
+    public static final int R_SECONDS = 0;
 
-    int accuracyMillis = -1;
+    /**
+     * Create time-stamps with a resolution of 1 tenth of a second.
+     */
+    public static final int R_TENTHS_OF_SECONDS = 1;
 
-    int accuracyMicros = -1;
+    /**
+     * Create time-stamps with a resolution of 1 microsecond.
+     */
+    public static final int R_MICROSECONDS = 2;
+
+    /**
+     * Create time-stamps with a resolution of 1 millisecond.
+     */
+    public static final int R_MILLISECONDS = 3;
+
+    private int resolution = R_SECONDS;
+    private Locale locale = null; // default locale
+
+    private int accuracySeconds = -1;
+
+    private int accuracyMillis = -1;
+
+    private int accuracyMicros = -1;
 
     boolean ordering = false;
 
@@ -245,6 +274,27 @@ public class TimeStampTokenGenerator
         otherRevoc.put(otherRevocationInfoFormat, otherRevocationInfos.getMatches(null));
     }
 
+    /**
+     * Set the resolution of the time stamp - R_SECONDS (the default), R_TENTH_OF_SECONDS, R_MICROSECONDS, R_MILLISECONDS
+     *
+     * @param resolution resolution of timestamps to be produced.
+     */
+    public void setResolution(int resolution)
+    {
+        this.resolution = resolution;
+    }
+
+    /**
+     * Set a Locale for time creation - you may need to use this if the default locale
+     * doesn't use a Gregorian calender so that the GeneralizedTime produced is compatible with other ASN.1 implementations.
+     *
+     * @param locale a locale to use for converting system time into a GeneralizedTime.
+     */
+    public void setLocale(Locale locale)
+    {
+        this.locale = locale;
+    }
+
     public void setAccuracySeconds(int accuracySeconds)
     {
         this.accuracySeconds = accuracySeconds;
@@ -284,11 +334,31 @@ public class TimeStampTokenGenerator
         BigInteger          serialNumber,
         Date                genTime)
         throws TSPException
+    {
+        return generate(request, serialNumber, genTime, null);
+    }
+
+    /**
+     * Generate a TimeStampToken for the passed in request and serialNumber marking it with the passed in genTime.
+     *
+     * @param request the originating request.
+     * @param serialNumber serial number for the TimeStampToken
+     * @param genTime token generation time.
+     * @param additionalExtensions extra extensions to be added to the response token.
+     * @return a TimeStampToken
+     * @throws TSPException
+     */
+    public TimeStampToken generate(
+        TimeStampRequest    request,
+        BigInteger          serialNumber,
+        Date                genTime,
+        Extensions          additionalExtensions)
+        throws TSPException
     {
         ASN1ObjectIdentifier digestAlgOID = request.getMessageImprintAlgOID();
 
         AlgorithmIdentifier algID = new AlgorithmIdentifier(digestAlgOID, DERNull.INSTANCE);
-        MessageImprint      messageImprint = new MessageImprint(algID, request.getMessageImprintDigest());
+        MessageImprint messageImprint = new MessageImprint(algID, request.getMessageImprintDigest());
 
         Accuracy accuracy = null;
         if (accuracySeconds > 0 || accuracyMillis > 0 || accuracyMicros > 0)
@@ -317,10 +387,10 @@ public class TimeStampTokenGenerator
         ASN1Boolean derOrdering = null;
         if (ordering)
         {
-            derOrdering = new ASN1Boolean(ordering);
+            derOrdering = ASN1Boolean.getInstance(ordering);
         }
 
-        ASN1Integer  nonce = null;
+        ASN1Integer nonce = null;
         if (request.getNonce() != null)
         {
             nonce = new ASN1Integer(request.getNonce());
@@ -332,10 +402,40 @@ public class TimeStampTokenGenerator
             tsaPolicy = request.getReqPolicy();
         }
 
+        Extensions respExtensions = request.getExtensions();
+        if (additionalExtensions != null)
+        {
+            ExtensionsGenerator extGen = new ExtensionsGenerator();
+
+            if (respExtensions != null)
+            {
+                for (Enumeration en = respExtensions.oids(); en.hasMoreElements(); )
+                {
+                    extGen.addExtension(respExtensions.getExtension(ASN1ObjectIdentifier.getInstance(en.nextElement())));
+                }
+            }
+            for (Enumeration en = additionalExtensions.oids(); en.hasMoreElements(); )
+            {
+                extGen.addExtension(additionalExtensions.getExtension(ASN1ObjectIdentifier.getInstance(en.nextElement())));
+            }
+
+            respExtensions = extGen.generate();
+        }
+
+        ASN1GeneralizedTime timeStampTime;
+        if (resolution == R_SECONDS)
+        {
+            timeStampTime = (locale == null) ? new ASN1GeneralizedTime(genTime) : new ASN1GeneralizedTime(genTime, locale);
+        }
+        else
+        {
+            timeStampTime = createGeneralizedTime(genTime);
+        }
+
         TSTInfo tstInfo = new TSTInfo(tsaPolicy,
                 messageImprint, new ASN1Integer(serialNumber),
-                new ASN1GeneralizedTime(genTime), accuracy, derOrdering,
-                nonce, tsa, request.getExtensions());
+                timeStampTime, accuracy, derOrdering,
+                nonce, tsa, respExtensions);
 
         try
         {
@@ -377,4 +477,59 @@ public class TimeStampTokenGenerator
             throw new TSPException("Exception encoding info", e);
         }
     }
+
+    // we need to produce a correct DER encoding GeneralizedTime here as the BC ASN.1 library doesn't handle this properly yet.
+    private ASN1GeneralizedTime createGeneralizedTime(Date time)
+        throws TSPException
+    {
+        String format = "yyyyMMddHHmmss.SSS";
+        SimpleDateFormat dateF = (locale == null) ? new SimpleDateFormat(format) : new SimpleDateFormat(format, locale);
+        dateF.setTimeZone(new SimpleTimeZone(0, "Z"));
+        StringBuilder sBuild = new StringBuilder(dateF.format(time));
+        int dotIndex = sBuild.indexOf(".");
+
+        if (dotIndex < 0)
+        {
+            // came back in seconds only, just return
+            sBuild.append("Z");
+            return new ASN1GeneralizedTime(sBuild.toString());
+        }
+
+        // trim to resolution
+        switch (resolution)
+        {
+        case R_TENTHS_OF_SECONDS:
+            if (sBuild.length() > dotIndex + 2)
+            {
+                sBuild.delete(dotIndex + 2, sBuild.length());
+            }
+            break;
+        case R_MICROSECONDS:
+            if (sBuild.length() > dotIndex + 3)
+            {
+                sBuild.delete(dotIndex + 3, sBuild.length());
+            }
+            break;
+        case R_MILLISECONDS:
+            // do nothing
+            break;
+        default:
+            throw new TSPException("unknown time-stamp resolution: " + resolution);
+        }
+
+        // remove trailing zeros
+        while (sBuild.charAt(sBuild.length() - 1) == '0')
+        {
+            sBuild.deleteCharAt(sBuild.length() - 1);
+        }
+
+        if (sBuild.length() - 1 == dotIndex)
+        {
+            sBuild.deleteCharAt(sBuild.length() - 1);
+        }
+
+        sBuild.append("Z");
+
+        return new ASN1GeneralizedTime(sBuild.toString());
+    }
 }
