@@ -505,13 +505,13 @@ public class TimeStampTokenGenerator
         case R_TENTHS_OF_SECONDS:
             if (sBuild.length() > dotIndex + 2)
             {
-                sBuild.delete(dotIndex + 2, sBuild.length());
+                sBuild = new StringBuffer(sBuild.toString().substring(0, dotIndex + 2));
             }
             break;
         case R_MICROSECONDS:
             if (sBuild.length() > dotIndex + 3)
             {
-                sBuild.delete(dotIndex + 3, sBuild.length());
+                sBuild = new StringBuffer(sBuild.toString().substring(0, dotIndex + 3));
             }
             break;
         case R_MILLISECONDS:
@@ -524,12 +524,12 @@ public class TimeStampTokenGenerator
         // remove trailing zeros
         while (sBuild.charAt(sBuild.length() - 1) == '0')
         {
-            sBuild.deleteCharAt(sBuild.length() - 1);
+            sBuild = new StringBuffer(sBuild.toString().substring(0, sBuild.length() - 1));
         }
 
         if (sBuild.length() - 1 == dotIndex)
         {
-            sBuild.deleteCharAt(sBuild.length() - 1);
+            sBuild = new StringBuffer(sBuild.toString().substring(0, sBuild.length() - 1));
         }
 
         sBuild.append("Z");
