@@ -1,5 +1,7 @@
 package org.bouncycastle.cms.test;
 
+import java.io.IOException;
+import java.security.AlgorithmParameters;
 import java.security.KeyPair;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
@@ -11,12 +13,23 @@ import java.util.Iterator;
 
 import javax.crypto.SecretKey;
 
+import junit.framework.Assert;
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.ASN1Primitive;
+import org.bouncycastle.asn1.DERNull;
+import org.bouncycastle.asn1.DEROctetString;
+import org.bouncycastle.asn1.cms.AuthenticatedData;
+import org.bouncycastle.asn1.cms.CCMParameters;
+import org.bouncycastle.asn1.cms.CMSObjectIdentifiers;
+import org.bouncycastle.asn1.cms.ContentInfo;
+import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
 import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
+import org.bouncycastle.asn1.util.ASN1Dump;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cms.CMSAlgorithm;
@@ -49,23 +62,23 @@ public class NewAuthenticatedDataTest
 {
     private static final String BC = BouncyCastleProvider.PROVIDER_NAME;
 
-    private static String          _signDN;
+    private static String _signDN;
     private static KeyPair _signKP;
     private static X509Certificate _signCert;
 
-    private static String          _origDN;
-    private static KeyPair         _origKP;
+    private static String _origDN;
+    private static KeyPair _origKP;
     private static X509Certificate _origCert;
 
-    private static String          _reciDN;
-    private static KeyPair         _reciKP;
+    private static String _reciDN;
+    private static KeyPair _reciKP;
     private static X509Certificate _reciCert;
 
-    private static KeyPair         _origEcKP;
-    private static KeyPair         _reciEcKP;
+    private static KeyPair _origEcKP;
+    private static KeyPair _reciEcKP;
     private static X509Certificate _reciEcCert;
 
-    private static boolean         _initialised = false;
+    private static boolean _initialised = false;
 
     public boolean DEBUG = true;
 
@@ -77,16 +90,16 @@ public class NewAuthenticatedDataTest
             _initialised = true;
             Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
 
-            _signDN   = "O=Bouncy Castle, C=AU";
-            _signKP   = CMSTestUtil.makeKeyPair();
+            _signDN = "O=Bouncy Castle, C=AU";
+            _signKP = CMSTestUtil.makeKeyPair();
             _signCert = CMSTestUtil.makeCertificate(_signKP, _signDN, _signKP, _signDN);
 
-            _origDN   = "CN=Bob, OU=Sales, O=Bouncy Castle, C=AU";
-            _origKP   = CMSTestUtil.makeKeyPair();
+            _origDN = "CN=Bob, OU=Sales, O=Bouncy Castle, C=AU";
+            _origKP = CMSTestUtil.makeKeyPair();
             _origCert = CMSTestUtil.makeCertificate(_origKP, _origDN, _signKP, _signDN);
 
-            _reciDN   = "CN=Doug, OU=Sales, O=Bouncy Castle, C=AU";
-            _reciKP   = CMSTestUtil.makeKeyPair();
+            _reciDN = "CN=Doug, OU=Sales, O=Bouncy Castle, C=AU";
+            _reciKP = CMSTestUtil.makeKeyPair();
             _reciCert = CMSTestUtil.makeCertificate(_reciKP, _reciDN, _signKP, _signDN);
 
             _origEcKP = CMSTestUtil.makeEcDsaKeyPair();
@@ -141,6 +154,9 @@ public class NewAuthenticatedDataTest
         throws Exception
     {
         tryKekAlgorithm(CMSTestUtil.makeDesede192Key(), new ASN1ObjectIdentifier("1.2.840.113549.1.9.16.3.6"));
+
+        DEROctetString iv = new DEROctetString(Hex.decode("0001020304050607"));
+        tryKekAlgorithm(CMSTestUtil.makeDesede192Key(), new ASN1ObjectIdentifier("1.2.840.113549.1.9.16.3.6"), iv.getEncoded());
     }
 
     public void testKEKDESedeWithDigest()
@@ -169,20 +185,20 @@ public class NewAuthenticatedDataTest
         adGen.addRecipientInfoGenerator(recipientGenerator);
 
         CMSAuthenticatedData ad = adGen.generate(
-                              new CMSProcessableByteArray(data),
-                              new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
 
-        RecipientInformationStore  recipients = ad.getRecipientInfos();
+        RecipientInformationStore recipients = ad.getRecipientInfos();
 
         assertEquals(ad.getMacAlgOID(),
-                CMSAuthenticatedDataGenerator.DES_EDE3_CBC);
+            CMSAuthenticatedDataGenerator.DES_EDE3_CBC);
 
-        Collection  c = recipients.getRecipients();
-        Iterator    it = c.iterator();
+        Collection c = recipients.getRecipients();
+        Iterator it = c.iterator();
 
         if (it.hasNext())
         {
-            RecipientInformation   recipient = (RecipientInformation)it.next();
+            RecipientInformation recipient = (RecipientInformation)it.next();
 
             byte[] recData = recipient.getContent(new JceKeyAgreeAuthenticatedRecipient(_reciEcKP.getPrivate()).setProvider(BC));
             assertTrue(Arrays.equals(data, recData));
@@ -197,18 +213,18 @@ public class NewAuthenticatedDataTest
     public void testEncoding()
         throws Exception
     {
-        byte[]          data     = "Eric H. Echidna".getBytes();
+        byte[] data = "Eric H. Echidna".getBytes();
 
         CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
 
         adGen.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(_reciCert).setProvider(BC));
 
         CMSAuthenticatedData ad = adGen.generate(
-                                new CMSProcessableByteArray(data),
-                                new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
 
         ad = new CMSAuthenticatedData(ad.getEncoded());
-        
+
         RecipientInformationStore recipients = ad.getRecipientInfos();
 
         assertEquals(CMSAuthenticatedDataGenerator.DES_EDE3_CBC, ad.getMacAlgOID());
@@ -235,7 +251,7 @@ public class NewAuthenticatedDataTest
     public void testOriginatorInfo()
         throws Exception
     {
-        byte[]               data = "Eric H. Echidna".getBytes();
+        byte[] data = "Eric H. Echidna".getBytes();
         ASN1ObjectIdentifier macAlg = CMSAlgorithm.DES_EDE3_CBC;
 
         CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
@@ -247,8 +263,8 @@ public class NewAuthenticatedDataTest
         adGen.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(_reciCert).setProvider(BC));
 
         CMSAuthenticatedData ad = adGen.generate(
-                                new CMSProcessableByteArray(data),
-                                new JceCMSMacCalculatorBuilder(macAlg).setProvider(BC).build());
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(macAlg).setProvider(BC).build());
 
         assertTrue(ad.getOriginatorInfo().getCertificates().getMatches(null).contains(origCert));
 
@@ -275,18 +291,150 @@ public class NewAuthenticatedDataTest
         }
     }
 
+    public void testAES256CCM()
+        throws Exception
+    {
+        byte[] data = "Eric H. Echidna".getBytes();
+        ASN1ObjectIdentifier macAlg = CMSAlgorithm.AES256_CCM;
+        AlgorithmParameters algParams = AlgorithmParameters.getInstance("CCM", BC);
+
+        algParams.init(new CCMParameters(Hex.decode("000102030405060708090a0b"), 16).getEncoded());
+
+        CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
+
+        X509CertificateHolder origCert = new X509CertificateHolder(_origCert.getEncoded());
+
+        adGen.setOriginatorInfo(new OriginatorInfoGenerator(origCert).generate());
+
+        adGen.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(_reciCert).setProvider(BC));
+
+        CMSAuthenticatedData ad = adGen.generate(
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(macAlg).setAlgorithmParameters(algParams).setProvider(BC).build());
+
+        assertTrue(ad.getOriginatorInfo().getCertificates().getMatches(null).contains(origCert));
+
+        RecipientInformationStore recipients = ad.getRecipientInfos();
+
+        assertEquals(ad.getMacAlgOID(), macAlg.getId());
+
+        Collection c = recipients.getRecipients();
+
+        assertEquals(1, c.size());
+
+        Iterator it = c.iterator();
+
+        while (it.hasNext())
+        {
+            RecipientInformation recipient = (RecipientInformation)it.next();
+
+            assertEquals(recipient.getKeyEncryptionAlgOID(), PKCSObjectIdentifiers.rsaEncryption.getId());
+
+            byte[] recData = recipient.getContent(new JceKeyTransAuthenticatedRecipient(_reciKP.getPrivate()).setProvider(BC));
+
+            assertTrue(Arrays.equals(data, recData));
+            assertEquals(16, ad.getMac().length);
+            assertTrue(Arrays.equals(ad.getMac(), recipient.getMac()));
+        }
+    }
+
+    public void testCMSAlgorithmProtection()
+        throws Exception
+    {
+        byte[] data = "Eric H. Echidna".getBytes();
+
+        CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
+        DigestCalculatorProvider calcProvider = new JcaDigestCalculatorProviderBuilder().setProvider(BC).build();
+
+        byte[] kekId = new byte[]{1, 2, 3, 4, 5};
+        SecretKey kek = CMSTestUtil.makeDesede192Key();
+
+        adGen.addRecipientInfoGenerator(new JceKEKRecipientInfoGenerator(kekId, kek).setProvider(BC));
+
+        CMSAuthenticatedData ad = adGen.generate(
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build(),
+            calcProvider.get(new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1)));
+
+        checkData(data, kek, ad);
+
+        ContentInfo adInfo = ad.toASN1Structure();
+        AuthenticatedData iAd = AuthenticatedData.getInstance(adInfo.getContent().toASN1Primitive().getEncoded());
+
+        try
+        {
+            new CMSAuthenticatedData(new ContentInfo(CMSObjectIdentifiers.authenticatedData,
+                        new AuthenticatedData(iAd.getOriginatorInfo(), iAd.getRecipientInfos(), iAd.getMacAlgorithm(), new AlgorithmIdentifier(TeleTrusTObjectIdentifiers.ripemd160, DERNull.INSTANCE), iAd.getEncapsulatedContentInfo(), iAd.getAuthAttrs(), iAd.getMac(), iAd.getUnauthAttrs())), calcProvider);
+        }
+        catch (CMSException e)
+        {
+            Assert.assertEquals(e.getMessage(), "CMS Algorithm Identifier Protection check failed for digestAlgorithm");
+        }
+
+        AlgorithmIdentifier newDigAlgId = new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1, DERNull.INSTANCE);
+        Assert.assertFalse(iAd.getDigestAlgorithm().equals(newDigAlgId));
+        checkData(data, kek, new CMSAuthenticatedData(new ContentInfo(CMSObjectIdentifiers.authenticatedData,
+            new AuthenticatedData(iAd.getOriginatorInfo(), iAd.getRecipientInfos(), iAd.getMacAlgorithm(), newDigAlgId, iAd.getEncapsulatedContentInfo(), iAd.getAuthAttrs(), iAd.getMac(), iAd.getUnauthAttrs())), calcProvider));
+
+        try
+        {
+            new CMSAuthenticatedData(new ContentInfo(CMSObjectIdentifiers.authenticatedData,
+                        new AuthenticatedData(iAd.getOriginatorInfo(), iAd.getRecipientInfos(), new AlgorithmIdentifier(CMSAlgorithm.AES192_CBC), iAd.getDigestAlgorithm(), iAd.getEncapsulatedContentInfo(), iAd.getAuthAttrs(), iAd.getMac(), iAd.getUnauthAttrs())), calcProvider);
+        }
+        catch (CMSException e)
+        {
+            Assert.assertEquals(e.getMessage(), "CMS Algorithm Identifier Protection check failed for macAlgorithm");
+        }
+
+        try
+        {
+            AlgorithmIdentifier newMacAlgId = new AlgorithmIdentifier(CMSAlgorithm.DES_EDE3_CBC);
+            Assert.assertFalse(iAd.getMacAlgorithm().equals(newMacAlgId));
+            new CMSAuthenticatedData(new ContentInfo(CMSObjectIdentifiers.authenticatedData,
+                new AuthenticatedData(iAd.getOriginatorInfo(), iAd.getRecipientInfos(), newMacAlgId, iAd.getDigestAlgorithm(), iAd.getEncapsulatedContentInfo(), iAd.getAuthAttrs(), iAd.getMac(), iAd.getUnauthAttrs())), calcProvider);
+        }
+        catch (CMSException e)
+        {
+            Assert.assertEquals(e.getMessage(), "CMS Algorithm Identifier Protection check failed for macAlgorithm");
+        }
+    }
+
+    private void checkData(byte[] data, SecretKey kek, CMSAuthenticatedData ad)
+        throws CMSException
+    {
+        RecipientInformationStore recipients = ad.getRecipientInfos();
+
+        Collection c = recipients.getRecipients();
+        Iterator it = c.iterator();
+
+        if (it.hasNext())
+        {
+            RecipientInformation recipient = (RecipientInformation)it.next();
+
+            byte[] recData = recipient.getContent(new JceKEKAuthenticatedRecipient(kek).setProvider(BC));
+
+            assertTrue(Arrays.equals(data, recData));
+            assertTrue(Arrays.equals(ad.getMac(), recipient.getMac()));
+            assertTrue(Arrays.equals(ad.getContentDigest(), recipient.getContentDigest()));
+        }
+        else
+        {
+            fail("no recipient found");
+        }
+    }
+
     private void tryKeyTrans(ASN1ObjectIdentifier macAlg)
         throws Exception
     {
-        byte[]          data     = "Eric H. Echidna".getBytes();
+        byte[] data = "Eric H. Echidna".getBytes();
 
         CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
 
         adGen.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(_reciCert).setProvider(BC));
-        
+
         CMSAuthenticatedData ad = adGen.generate(
-                                new CMSProcessableByteArray(data),
-                                new JceCMSMacCalculatorBuilder(macAlg).setProvider(BC).build());
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(macAlg).setProvider(BC).build());
 
         RecipientInformationStore recipients = ad.getRecipientInfos();
 
@@ -314,7 +462,7 @@ public class NewAuthenticatedDataTest
     private void tryKeyTransWithDigest(ASN1ObjectIdentifier macAlg)
         throws Exception
     {
-        byte[]          data     = "Eric H. Echidna".getBytes();
+        byte[] data = "Eric H. Echidna".getBytes();
 
         CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
         DigestCalculatorProvider calcProvider = new JcaDigestCalculatorProviderBuilder().setProvider(BC).build();
@@ -322,9 +470,9 @@ public class NewAuthenticatedDataTest
         adGen.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(_reciCert).setProvider(BC));
 
         CMSAuthenticatedData ad = adGen.generate(
-                                new CMSProcessableByteArray(data),
-                                new JceCMSMacCalculatorBuilder(macAlg).setProvider(BC).build(),
-                                calcProvider.get(new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1)));
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(macAlg).setProvider(BC).build(),
+            calcProvider.get(new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1)));
 
         RecipientInformationStore recipients = ad.getRecipientInfos();
 
@@ -353,17 +501,17 @@ public class NewAuthenticatedDataTest
     private void tryKekAlgorithm(SecretKey kek, ASN1ObjectIdentifier algOid)
         throws NoSuchAlgorithmException, NoSuchProviderException, CMSException, OperatorCreationException
     {
-        byte[]          data     = "Eric H. Echidna".getBytes();
+        byte[] data = "Eric H. Echidna".getBytes();
 
         CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
-       
-        byte[]  kekId = new byte[] { 1, 2, 3, 4, 5 };
+
+        byte[] kekId = new byte[]{1, 2, 3, 4, 5};
 
         adGen.addRecipientInfoGenerator(new JceKEKRecipientInfoGenerator(kekId, kek).setProvider(BC));
 
         CMSAuthenticatedData ad = adGen.generate(
-                                new CMSProcessableByteArray(data),
-                                new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
 
         RecipientInformationStore recipients = ad.getRecipientInfos();
 
@@ -389,48 +537,91 @@ public class NewAuthenticatedDataTest
         }
     }
 
+    private void tryKekAlgorithm(SecretKey kek, ASN1ObjectIdentifier algOid, byte[] encodedParameters)
+        throws NoSuchAlgorithmException, NoSuchProviderException, CMSException, OperatorCreationException, IOException
+    {
+        byte[] data = "Eric H. Echidna".getBytes();
+
+        CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
+
+        byte[] kekId = new byte[]{1, 2, 3, 4, 5};
+
+        adGen.addRecipientInfoGenerator(new JceKEKRecipientInfoGenerator(kekId, kek).setProvider(BC));
+
+        AlgorithmParameters algParams = AlgorithmParameters.getInstance(CMSAlgorithm.DES_EDE3_CBC.getId(), "BC");
+
+        algParams.init(encodedParameters);
+
+        CMSAuthenticatedData ad = adGen.generate(
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setAlgorithmParameters(algParams).setProvider(BC).build());
+
+        RecipientInformationStore recipients = ad.getRecipientInfos();
+
+        Collection c = recipients.getRecipients();
+        Iterator it = c.iterator();
+
+        assertEquals(ad.getMacAlgOID(), CMSAuthenticatedDataGenerator.DES_EDE3_CBC);
+        assertEquals(ad.getMacAlgorithm().getParameters(), ASN1Primitive.fromByteArray(encodedParameters));
+
+        if (it.hasNext())
+        {
+            RecipientInformation recipient = (RecipientInformation)it.next();
+
+            assertEquals(recipient.getKeyEncryptionAlgOID(), algOid.getId());
+
+            byte[] recData = recipient.getContent(new JceKEKAuthenticatedRecipient(kek).setProvider(BC));
+
+            assertTrue(Arrays.equals(data, recData));
+            assertTrue(Arrays.equals(ad.getMac(), recipient.getMac()));
+        }
+        else
+        {
+            fail("no recipient found");
+        }
+    }
+
     private void tryKekAlgorithmWithDigest(SecretKey kek, ASN1ObjectIdentifier algOid)
         throws NoSuchAlgorithmException, NoSuchProviderException, CMSException, OperatorCreationException
     {
-            byte[]          data     = "Eric H. Echidna".getBytes();
+        byte[] data = "Eric H. Echidna".getBytes();
 
-            CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
-            DigestCalculatorProvider calcProvider = new JcaDigestCalculatorProviderBuilder().setProvider(BC).build();
+        CMSAuthenticatedDataGenerator adGen = new CMSAuthenticatedDataGenerator();
+        DigestCalculatorProvider calcProvider = new JcaDigestCalculatorProviderBuilder().setProvider(BC).build();
 
-            byte[]  kekId = new byte[] { 1, 2, 3, 4, 5 };
+        byte[] kekId = new byte[]{1, 2, 3, 4, 5};
 
-            adGen.addRecipientInfoGenerator(new JceKEKRecipientInfoGenerator(kekId, kek).setProvider(BC));
+        adGen.addRecipientInfoGenerator(new JceKEKRecipientInfoGenerator(kekId, kek).setProvider(BC));
 
-            CMSAuthenticatedData ad = adGen.generate(
-                                    new CMSProcessableByteArray(data),
-                                    new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build(),
-                                    calcProvider.get(new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1)));
+        CMSAuthenticatedData ad = adGen.generate(
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build(),
+            calcProvider.get(new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1)));
 
-            RecipientInformationStore recipients = ad.getRecipientInfos();
+        RecipientInformationStore recipients = ad.getRecipientInfos();
 
-            Collection c = recipients.getRecipients();
-            Iterator it = c.iterator();
+        Collection c = recipients.getRecipients();
+        Iterator it = c.iterator();
 
-            assertEquals(ad.getMacAlgOID(), CMSAuthenticatedDataGenerator.DES_EDE3_CBC);
+        assertEquals(ad.getMacAlgOID(), CMSAuthenticatedDataGenerator.DES_EDE3_CBC);
 
-            if (it.hasNext())
-            {
-                RecipientInformation recipient = (RecipientInformation)it.next();
+        if (it.hasNext())
+        {
+            RecipientInformation recipient = (RecipientInformation)it.next();
 
-                assertEquals(recipient.getKeyEncryptionAlgOID(), algOid.getId());
+            assertEquals(recipient.getKeyEncryptionAlgOID(), algOid.getId());
 
-                byte[] recData = recipient.getContent(new JceKEKAuthenticatedRecipient(kek).setProvider(BC));
+            byte[] recData = recipient.getContent(new JceKEKAuthenticatedRecipient(kek).setProvider(BC));
 
-                assertTrue(Arrays.equals(data, recData));
-                assertTrue(Arrays.equals(ad.getMac(), recipient.getMac()));
-                assertTrue(Arrays.equals(ad.getContentDigest(), recipient.getContentDigest()));
-            }
-            else
-            {
-                fail("no recipient found");
-            }
+            assertTrue(Arrays.equals(data, recData));
+            assertTrue(Arrays.equals(ad.getMac(), recipient.getMac()));
+            assertTrue(Arrays.equals(ad.getContentDigest(), recipient.getContentDigest()));
         }
-
+        else
+        {
+            fail("no recipient found");
+        }
+    }
 
     private void passwordTest(String algorithm)
         throws Exception
@@ -442,16 +633,16 @@ public class NewAuthenticatedDataTest
         adGen.addRecipientInfoGenerator(new JcePasswordRecipientInfoGenerator(new ASN1ObjectIdentifier(algorithm), "password".toCharArray()).setProvider(BC).setSaltAndIterationCount(new byte[20], 5));
 
         CMSAuthenticatedData ad = adGen.generate(
-                              new CMSProcessableByteArray(data),
-                              new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
+            new CMSProcessableByteArray(data),
+            new JceCMSMacCalculatorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider(BC).build());
 
-        RecipientInformationStore  recipients = ad.getRecipientInfos();
+        RecipientInformationStore recipients = ad.getRecipientInfos();
 
         assertEquals(ad.getMacAlgOID(),
-                                   CMSAuthenticatedDataGenerator.DES_EDE3_CBC);
+            CMSAuthenticatedDataGenerator.DES_EDE3_CBC);
 
-        Collection  c = recipients.getRecipients();
-        Iterator    it = c.iterator();
+        Collection c = recipients.getRecipients();
+        Iterator it = c.iterator();
 
         if (it.hasNext())
         {
