@@ -9,19 +9,19 @@ import java.security.NoSuchProviderException;
 import java.security.PublicKey;
 import java.security.Security;
 import java.security.SignatureException;
-import org.bouncycastle.jce.cert.CertPath;
-import org.bouncycastle.jce.cert.CertPathBuilder;
-import org.bouncycastle.jce.cert.CertPathBuilderException;
-import org.bouncycastle.jce.cert.CertPathBuilderResult;
-import org.bouncycastle.jce.cert.CertStore;
+import java.security.cert.CertPath;
+import java.security.cert.CertPathBuilder;
+import java.security.cert.CertPathBuilderException;
+import java.security.cert.CertPathBuilderResult;
+import java.security.cert.CertStore;
 import java.security.cert.Certificate;
 import java.security.cert.CertificateEncodingException;
 import java.security.cert.CertificateException;
-import org.bouncycastle.jce.cert.CertificateFactory;
-import org.bouncycastle.jce.cert.CollectionCertStoreParameters;
-import org.bouncycastle.jce.cert.PKIXBuilderParameters;
-import org.bouncycastle.jce.cert.TrustAnchor;
-import org.bouncycastle.jce.cert.X509CertSelector;
+import java.security.cert.CertificateFactory;
+import java.security.cert.CollectionCertStoreParameters;
+import java.security.cert.PKIXBuilderParameters;
+import java.security.cert.TrustAnchor;
+import java.security.cert.X509CertSelector;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -31,7 +31,7 @@ import java.util.List;
 import java.util.Set;
 import java.util.Vector;
 
-import org.bouncycastle.jce.X509Principal;
+import org.bouncycastle.jce.PrincipalUtil;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.util.encoders.Base64;
 import org.bouncycastle.util.test.SimpleTest;
@@ -162,7 +162,7 @@ public class CertPathTest
 
         CertPathBuilder pathBuilder = CertPathBuilder.getInstance("PKIX", "BC");
         X509CertSelector select = new X509CertSelector();
-        select.setSubject(((X509Principal)((X509Certificate)certCol.get(0)).getSubjectDN()).getEncoded());
+        select.setSubject(PrincipalUtil.getSubjectX509Principal(((X509Certificate)certCol.get(0))).getEncoded());
 
         Set trustanchors = new HashSet();
         trustanchors.add(new TrustAnchor((X509Certificate)cf.generateCertificate(new ByteArrayInputStream(rootCertBin)), null));
@@ -324,9 +324,9 @@ public class CertPathTest
 
     private static class MyCertPath extends CertPath
     {
-        private final Vector certificates;
+        private final List certificates;
 
-        private final Vector encodingNames;
+        private final List encodingNames;
 
         private final byte[] encoding;
 
@@ -334,9 +334,9 @@ public class CertPathTest
         {
             super("MyEncoding");
             this.encoding = encoding;
-            certificates = new Vector();
+            certificates = new ArrayList();
             certificates.add(new MyCertificate("MyEncoding", encoding));
-            encodingNames = new Vector();
+            encodingNames = new ArrayList();
             encodingNames.add("MyEncoding");
         }
 
