@@ -12,9 +12,9 @@ import java.security.SignatureException;
 import java.security.interfaces.RSAPrivateKey;
 import java.security.interfaces.RSAPublicKey;
 import java.security.spec.AlgorithmParameterSpec;
-import java.security.spec.PSSParameterSpec;
 
 import org.bouncycastle.crypto.AsymmetricBlockCipher;
+import org.bouncycastle.crypto.CipherParameters;
 import org.bouncycastle.crypto.CryptoException;
 import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.digests.SHA1Digest;
@@ -26,25 +26,20 @@ import org.bouncycastle.crypto.digests.SHA512tDigest;
 import org.bouncycastle.crypto.digests.SHA3Digest;
 import org.bouncycastle.crypto.engines.RSABlindedEngine;
 import org.bouncycastle.crypto.params.ParametersWithRandom;
-import org.bouncycastle.jce.provider.BouncyCastleProvider;
-import org.bouncycastle.jcajce.util.BCJcaJceHelper;
-import org.bouncycastle.jcajce.util.JcaJceHelper;
 
 public class PSSSignatureSpi
     extends Signature
 {
-    private final JcaJceHelper helper = new BCJcaJceHelper();
-
     private AlgorithmParameters engineParams;
-    private PSSParameterSpec paramSpec;
     private AsymmetricBlockCipher signer;
     private Digest contentDigest;
     private Digest mgfDigest;
     private int saltLength;
     private byte trailer;
     private boolean isRaw;
-
+    private ByteArrayOutputStream bOut;
     private org.bouncycastle.crypto.signers.PSSSigner pss;
+    private CipherParameters sigParams;
 
     private byte getTrailer(
         int trailerField)
@@ -88,10 +83,6 @@ public class PSSSignatureSpi
             this.saltLength = 20;
         }
 
-        if (paramSpec != null)
-        {
-            this.saltLength = paramSpec.getSaltLength();
-        }
         this.isRaw = false;
 
         setupContentDigest();
@@ -118,11 +109,6 @@ public class PSSSignatureSpi
             this.saltLength = 20;
         }
 
-        if (paramSpec != null)
-        {
-            this.saltLength = paramSpec.getSaltLength();
-        }
-
         this.isRaw = isRaw;
 
         setupContentDigest();
@@ -137,9 +123,24 @@ public class PSSSignatureSpi
             throw new InvalidKeyException("Supplied key is not a RSAPublicKey instance");
         }
 
-        pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, contentDigest, mgfDigest, saltLength);
-        pss.init(false,
-            RSAUtil.generatePublicKeyParameter((RSAPublicKey)publicKey));
+        if (mgfDigest == null)
+        {
+            mgfDigest = new SHA1Digest();
+            setupContentDigest();
+        }
+
+        sigParams = RSAUtil.generatePublicKeyParameter((RSAPublicKey)publicKey);
+
+        if (isRaw)
+        {
+            bOut = new ByteArrayOutputStream();
+        }
+        else
+        {
+            pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, contentDigest, mgfDigest, saltLength);
+            pss.init(false,
+                sigParams);
+        }
     }
 
     protected void engineInitSign(
@@ -152,8 +153,23 @@ public class PSSSignatureSpi
             throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
         }
 
-        pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, contentDigest, mgfDigest, saltLength);
-        pss.init(true, new ParametersWithRandom(RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey), random));
+        sigParams = new ParametersWithRandom(RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey), random);
+
+        if (mgfDigest == null)
+        {
+            mgfDigest = new SHA1Digest();
+            setupContentDigest();
+        }
+
+        if (isRaw)
+        {
+            bOut = new ByteArrayOutputStream();
+        }
+        else
+        {
+            pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, contentDigest, mgfDigest, saltLength);
+            pss.init(true, sigParams);
+        }
     }
 
     protected void engineInitSign(
@@ -165,15 +181,37 @@ public class PSSSignatureSpi
             throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
         }
 
-        pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, contentDigest, mgfDigest, saltLength);
-        pss.init(true, RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey));
+        sigParams = RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey);
+
+        if (mgfDigest == null)
+        {
+            mgfDigest = new SHA1Digest();
+            setupContentDigest();
+        }
+        
+        if (isRaw)
+        {
+            bOut = new ByteArrayOutputStream();
+        }
+        else
+        {
+            pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, contentDigest, mgfDigest, saltLength);
+            pss.init(true, sigParams);
+        }
     }
 
     protected void engineUpdate(
         byte    b)
         throws SignatureException
     {
-        pss.update(b);
+        if (isRaw)
+        {
+            bOut.write(b);
+        }
+        else
+        {
+            pss.update(b);
+        }
     }
 
     protected void engineUpdate(
@@ -182,7 +220,14 @@ public class PSSSignatureSpi
         int     len) 
         throws SignatureException
     {
-        pss.update(b, off, len);
+        if (isRaw)
+        {
+            bOut.write(b, off, len);
+        }
+        else
+        {
+            pss.update(b, off, len);
+        }
     }
 
     protected byte[] engineSign()
@@ -190,6 +235,15 @@ public class PSSSignatureSpi
     {
         try
         {
+            if (isRaw)
+            {
+                byte[] hash = bOut.toByteArray();
+                contentDigest = mgfDigest = guessDigest(hash.length);
+                saltLength = contentDigest.getDigestSize();
+                pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, new NullPssDigest(contentDigest), mgfDigest, saltLength);
+
+                pss.init(true, sigParams);
+            }
             return pss.generateSignature();
         }
         catch (CryptoException e)
@@ -202,6 +256,17 @@ public class PSSSignatureSpi
         byte[]  sigBytes) 
         throws SignatureException
     {
+        if (isRaw)
+        {
+            byte[] hash = bOut.toByteArray();
+            contentDigest = mgfDigest = guessDigest(hash.length);
+            saltLength = contentDigest.getDigestSize();
+            pss = new org.bouncycastle.crypto.signers.PSSSigner(signer, new NullPssDigest(contentDigest), mgfDigest, saltLength);
+
+            pss.init(false, sigParams);
+
+            pss.update(hash, 0, hash.length);
+        }
         return pss.verifySignature(sigBytes);
     }
 
@@ -209,71 +274,11 @@ public class PSSSignatureSpi
         AlgorithmParameterSpec params)
         throws InvalidParameterException
     {
-        if (params instanceof PSSParameterSpec)
-        {
-            PSSParameterSpec newParamSpec = (PSSParameterSpec)params;
-
-            this.engineParams = null;
-            this.paramSpec = newParamSpec;
-            this.saltLength = paramSpec.getSaltLength();
-
-            if (mgfDigest == null)
-            {
-                switch (saltLength)
-                {
-                case 20:
-                    this.mgfDigest = new SHA1Digest();
-                    break;
-                case 28:
-                    this.mgfDigest = new SHA224Digest();
-                    break;
-                case 32:
-                    this.mgfDigest = new SHA256Digest();
-                    break;
-                case 48:
-                    this.mgfDigest = new SHA384Digest();
-                    break;
-                case 64:
-                    this.mgfDigest = new SHA512Digest();
-                    break;
-                default:
-                    if (saltLength <= 20)
-                    {
-                        this.mgfDigest = new SHA1Digest();
-                    }
-                    else if (saltLength <= 28)
-                    {
-                        this.mgfDigest = new SHA224Digest();
-                    }
-                    else if (saltLength <= 32)
-                    {
-                         this.mgfDigest = new SHA256Digest();
-                    }
-                }
-                setupContentDigest();
-            }
-        }
-        else
-        {
             throw new InvalidParameterException("Only PSSParameterSpec supported");
-        }
     }
 
     protected AlgorithmParameters engineGetParameters()
     {
-        if (engineParams == null)
-        {
-            try
-            {
-                engineParams = helper.createAlgorithmParameters("PSS");
-                engineParams.init(new PSSParameterSpec(saltLength));
-            }
-            catch (Exception e)
-            {
-                throw new RuntimeException(e.toString());
-            }
-        }
-
         return engineParams;
     }
     
@@ -293,6 +298,25 @@ public class PSSSignatureSpi
         throw new UnsupportedOperationException("engineGetParameter unsupported");
     }
 
+    private Digest guessDigest(int size)
+    {
+        switch (size)
+        {
+        case 20:
+            return new SHA1Digest();
+        case 28:
+            return new SHA224Digest();
+        case 32:
+            return new SHA256Digest();
+        case 48:
+            return new SHA384Digest();
+        case 64:
+            return new SHA512Digest();
+        }
+
+        return null;
+    }
+
     static public class nonePSS
         extends PSSSignatureSpi
     {
@@ -409,7 +433,6 @@ public class PSSSignatureSpi
             super("SHA3-512withRSAandMGF1", new RSABlindedEngine(), new SHA3Digest(512));
         }
     }
-
     private class NullPssDigest
         implements Digest
     {
