@@ -39,6 +39,7 @@ import org.bouncycastle.jce.spec.ECPrivateKeySpec;
 import org.bouncycastle.math.ec.ECCurve;
 import org.bouncycastle.math.ec.ECPoint;
 import org.bouncycastle.util.Strings;
+import org.bouncycastle.jcajce.provider.asymmetric.util.ECUtil;
 
 public class BCECGOST3410PrivateKey
     implements ECPrivateKey, PKCS12BagAttributeCarrier, ECPointEncoder
@@ -304,6 +305,11 @@ public class BCECGOST3410PrivateKey
         return BouncyCastleProvider.CONFIGURATION.getEcImplicitlyCa();
     }
 
+    public String toString()
+    {
+        return ECUtil.privateKeyToString(algorithm, d, engineGetSpec());
+    }
+
     public boolean equals(Object o)
     {
         if (!(o instanceof BCECGOST3410PrivateKey))
