@@ -289,6 +289,22 @@ public class CertPathValidatorUtilities
         return trust;
     }
 
+    static boolean isIssuerTrustAnchor(
+        X509Certificate cert,
+        Set trustAnchors,
+        String sigProvider)
+        throws AnnotatedException
+    {
+        try
+        {
+            return findTrustAnchor(cert, trustAnchors, sigProvider) != null;
+        }
+        catch (Exception e)
+        {
+            return false;
+        }
+    }
+    
     protected static void addAdditionalStoresFromAltNames(
         X509Certificate cert,
         ExtendedPKIXParameters pkixParams)
