@@ -1,34 +1,49 @@
 package org.bouncycastle.jsse.provider;
 
 import java.io.IOException;
+import java.lang.reflect.Constructor;
+import java.security.AccessController;
 import java.security.Principal;
+import java.security.PrivateKey;
+import java.security.PrivilegedAction;
 import java.security.cert.CertificateEncodingException;
 import java.security.cert.X509Certificate;
+import java.security.interfaces.DSAPrivateKey;
+import java.security.interfaces.ECPrivateKey;
+import java.security.interfaces.RSAPrivateKey;
+import java.util.Collection;
 import java.util.Collections;
+import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Set;
 import java.util.Vector;
 
+import javax.crypto.interfaces.DHPrivateKey;
 import javax.security.auth.x500.X500Principal;
 
 import org.bouncycastle.asn1.ASN1Encoding;
 import org.bouncycastle.asn1.x500.X500Name;
-import org.bouncycastle.jcajce.util.DefaultJcaJceHelper;
-import org.bouncycastle.jcajce.util.JcaJceHelper;
+import org.bouncycastle.jsse.BCSNIHostName;
+import org.bouncycastle.jsse.BCSNIMatcher;
+import org.bouncycastle.jsse.BCSNIServerName;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.AlertLevel;
 import org.bouncycastle.tls.Certificate;
 import org.bouncycastle.tls.ClientCertificateType;
 import org.bouncycastle.tls.HashAlgorithm;
 import org.bouncycastle.tls.KeyExchangeAlgorithm;
+import org.bouncycastle.tls.NameType;
+import org.bouncycastle.tls.ServerName;
+import org.bouncycastle.tls.ServerNameList;
 import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.SignatureAndHashAlgorithm;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.TlsCrypto;
 import org.bouncycastle.tls.crypto.impl.jcajce.JcaTlsCertificate;
+import org.bouncycastle.tls.crypto.impl.jcajce.JcaTlsCrypto;
 
-class JsseUtils
+abstract class JsseUtils
 {
     protected static X509Certificate[] EMPTY_CHAIN = new X509Certificate[0];
 
@@ -129,22 +144,19 @@ class JsseUtils
         return new Certificate(certificateList);
     }
 
-    public static X509Certificate[] getX509CertificateChain(Certificate certificateMessage)
+    public static X509Certificate[] getX509CertificateChain(TlsCrypto crypto, Certificate certificateMessage)
     {
         if (certificateMessage == null || certificateMessage.isEmpty())
         {
             return EMPTY_CHAIN;
         }
 
-        // TODO[jsse] Consider provider-related issues
-        JcaJceHelper helper = new DefaultJcaJceHelper();
-
         try
         {
             X509Certificate[] chain = new X509Certificate[certificateMessage.getLength()];
             for (int i = 0; i < chain.length; ++i)
             {
-                chain[i] = JcaTlsCertificate.convert(certificateMessage.getCertificateAt(i), helper).getX509Certificate();
+                chain[i] = JcaTlsCertificate.convert((JcaTlsCrypto)crypto, certificateMessage.getCertificateAt(i)).getX509Certificate();
             }
             return chain;
         }
@@ -178,19 +190,16 @@ class JsseUtils
         return x509Chain;
     }
 
-    public static X500Principal getSubject(Certificate certificateMessage)
+    public static X500Principal getSubject(TlsCrypto crypto, Certificate certificateMessage)
     {
         if (certificateMessage == null || certificateMessage.isEmpty())
         {
             return null;
         }
 
-        // TODO[jsse] Consider provider-related issues
-        JcaJceHelper helper = new DefaultJcaJceHelper();
-
         try
         {
-            return JcaTlsCertificate.convert(certificateMessage.getCertificateAt(0), helper).getX509Certificate()
+            return JcaTlsCertificate.convert((JcaTlsCrypto)crypto, certificateMessage.getCertificateAt(0)).getX509Certificate()
                 .getSubjectX500Principal();
         }
         catch (IOException e)
@@ -226,6 +235,39 @@ class JsseUtils
         return result;
     }
 
+    public static boolean isUsableKeyForServer(int keyExchangeAlgorithm, PrivateKey privateKey) throws IOException
+    {
+        switch (keyExchangeAlgorithm)
+        {
+        case KeyExchangeAlgorithm.DH_DSS:
+        case KeyExchangeAlgorithm.DH_DSS_EXPORT:
+        case KeyExchangeAlgorithm.DH_RSA:
+        case KeyExchangeAlgorithm.DH_RSA_EXPORT:
+            return privateKey instanceof DHPrivateKey;
+
+        case KeyExchangeAlgorithm.ECDH_ECDSA:
+        case KeyExchangeAlgorithm.ECDH_RSA:
+        case KeyExchangeAlgorithm.ECDHE_ECDSA:
+            return privateKey instanceof ECPrivateKey;
+
+        case KeyExchangeAlgorithm.DHE_DSS:
+        case KeyExchangeAlgorithm.DHE_DSS_EXPORT:
+        case KeyExchangeAlgorithm.SRP_DSS:
+            return privateKey instanceof DSAPrivateKey;
+
+        case KeyExchangeAlgorithm.DHE_RSA:
+        case KeyExchangeAlgorithm.DHE_RSA_EXPORT:
+        case KeyExchangeAlgorithm.ECDHE_RSA:
+        case KeyExchangeAlgorithm.RSA:
+        case KeyExchangeAlgorithm.RSA_PSK:
+        case KeyExchangeAlgorithm.SRP_RSA:
+            return privateKey instanceof RSAPrivateKey;
+
+        default:
+            return false;
+        }
+    }
+
     static Set<X500Principal> toX500Principals(X500Name[] names) throws IOException
     {
         if (names == null || names.length == 0)
@@ -292,4 +334,105 @@ class JsseUtils
             v.addElement(alg);
         }
     }
+
+    static Constructor getDeclaredConstructor(final Class clazz, final Class<?>... parameterTypes)
+    {
+        return AccessController.doPrivileged(new PrivilegedAction<Constructor>()
+        {
+            public Constructor run()
+            {
+                try
+                {
+                    return clazz.getDeclaredConstructor(parameterTypes);
+                }
+                catch (Exception e)
+                {
+                    // ignore - maybe log?
+                }
+
+                return null;
+            }
+        });
+    }
+
+    static Class loadClass(Class sourceClass, final String className)
+    {
+        try
+        {
+            ClassLoader loader = sourceClass.getClassLoader();
+            if (loader != null)
+            {
+                return loader.loadClass(className);
+            }
+            else
+            {
+                return AccessController.doPrivileged(new PrivilegedAction<Class>()
+                {
+                    public Class run()
+                    {
+                        try
+                        {
+                            return Class.forName(className);
+                        }
+                        catch (Exception e)
+                        {
+                            // ignore - maybe log?
+                        }
+
+                        return null;
+                    }
+                });
+            }
+        }
+        catch (ClassNotFoundException e)
+        {
+            // ignore - maybe log?
+        }
+
+        return null;
+    }
+
+    static BCSNIServerName convertSNIServerName(ServerName serverName)
+    {
+        switch (serverName.getNameType())
+        {
+        case NameType.host_name:
+            return new BCSNIHostName(serverName.getHostName());
+        default:
+            return null;
+        }
+    }
+
+    static BCSNIServerName findMatchingSNIServerName(ServerNameList serverNameList,
+        Collection<BCSNIMatcher> sniMatchers)
+    {
+        Enumeration serverNames = serverNameList.getServerNameList().elements();
+        while (serverNames.hasMoreElements())
+        {
+            BCSNIServerName sniServerName = convertSNIServerName((ServerName)serverNames.nextElement());
+
+            for (BCSNIMatcher sniMatcher : sniMatchers)
+            {
+                if (sniMatcher != null && sniMatcher.getType() == sniServerName.getType()
+                    && sniMatcher.matches(sniServerName))
+                {
+                    return sniServerName;
+                }
+            }
+        }
+        return null;
+    }
+
+    static String stripQuotes(String s)
+    {
+        if (s != null)
+        {
+            int sLast = s.length() - 1;
+            if (sLast > 0 && s.charAt(0) == '"' && s.charAt(sLast) == '"')
+            {
+                return s.substring(1, sLast);
+            }
+        }
+        return s;
+    }
 }
