@@ -1,15 +1,36 @@
 package org.bouncycastle.jsse.provider;
 
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.List;
+
+import org.bouncycastle.jsse.BCSNIMatcher;
+import org.bouncycastle.jsse.BCSNIServerName;
+
 class ProvSSLParameters
 {
     static final boolean hasSslParameters;
 
+    private static <T> List<T> copyList(List<T> list)
+    {
+        if (list == null)
+        {
+            return null;
+        }
+        if (list.isEmpty())
+        {
+            return Collections.emptyList();
+        }
+        return Collections.unmodifiableList(new ArrayList<T>(list));
+    }
+
     static
     {
-        Class clazz = null;
+        Class<?> clazz = null;
         try
         {
-            clazz = ProvSSLServerSocket.class.getClassLoader().loadClass("javax.net.ssl.SSLParameters");
+            clazz = JsseUtils.loadClass(ProvSSLParameters.class,"javax.net.ssl.SSLParameters");
         }
         catch (Exception e)
         {
@@ -26,6 +47,8 @@ class ProvSSLParameters
     private Object algorithmConstraints;      // object not introduced till 1.6
     private String endpointIdentificationAlgorithm;
     private boolean useCipherSuitesOrder;
+    private List<BCSNIMatcher> sniMatchers;
+    private List<BCSNIServerName> sniServerNames;
 
     public void setCipherSuites(String[] cipherSuites)
     {
@@ -98,6 +121,26 @@ class ProvSSLParameters
         this.useCipherSuitesOrder = honorOrder;
     }
 
+    public List<BCSNIServerName> getServerNames()
+    {
+        return copyList(sniServerNames);
+    }
+
+    public void setServerNames(List<BCSNIServerName> serverNames)
+    {
+        this.sniServerNames = copyList(serverNames);
+    }
+
+    public Collection<BCSNIMatcher> getSNIMatchers()
+    {
+        return copyList(sniMatchers);
+    }
+
+    public void setSNIMatchers(Collection<BCSNIMatcher> matchers)
+    {
+        throw new UnsupportedOperationException();
+    }
+
     static ProvSSLParameters extractDefaultParameters(ProvSSLContextSpi context)
     {
         if (hasSslParameters)
