@@ -1,26 +1,31 @@
 package org.bouncycastle.jsse.provider;
 
 import java.io.IOException;
+import java.io.UnsupportedEncodingException;
 import java.net.Socket;
 import java.security.Principal;
 import java.security.PrivateKey;
 import java.security.cert.X509Certificate;
 import java.util.Hashtable;
+import java.util.List;
 import java.util.Set;
 import java.util.Vector;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import javax.net.ssl.SSLSession;
 import javax.net.ssl.X509KeyManager;
 import javax.security.auth.x500.X500Principal;
 
 import org.bouncycastle.asn1.x500.X500Name;
+import org.bouncycastle.jsse.BCSNIServerName;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.AlertLevel;
 import org.bouncycastle.tls.Certificate;
 import org.bouncycastle.tls.CertificateRequest;
+import org.bouncycastle.tls.CertificateStatusRequest;
+import org.bouncycastle.tls.CompressionMethod;
 import org.bouncycastle.tls.DefaultTlsClient;
+import org.bouncycastle.tls.DefaultTlsKeyExchangeFactory;
 import org.bouncycastle.tls.KeyExchangeAlgorithm;
 import org.bouncycastle.tls.NameType;
 import org.bouncycastle.tls.ProtocolVersion;
@@ -31,6 +36,7 @@ import org.bouncycastle.tls.TlsAuthentication;
 import org.bouncycastle.tls.TlsCredentials;
 import org.bouncycastle.tls.TlsExtensionsUtils;
 import org.bouncycastle.tls.TlsFatalAlert;
+import org.bouncycastle.tls.TlsServerCertificate;
 import org.bouncycastle.tls.TlsSession;
 import org.bouncycastle.tls.TlsUtils;
 import org.bouncycastle.tls.crypto.TlsCrypto;
@@ -38,7 +44,9 @@ import org.bouncycastle.tls.crypto.TlsCryptoParameters;
 import org.bouncycastle.tls.crypto.impl.jcajce.JcaDefaultTlsCredentialedSigner;
 import org.bouncycastle.tls.crypto.impl.jcajce.JcaTlsCrypto;
 import org.bouncycastle.tls.crypto.impl.jcajce.JceDefaultTlsCredentialedAgreement;
+import org.bouncycastle.util.Arrays;
 import org.bouncycastle.util.IPAddress;
+import org.bouncycastle.util.encoders.Hex;
 
 class ProvTlsClient
     extends DefaultTlsClient
@@ -46,19 +54,83 @@ class ProvTlsClient
 {
     private static Logger LOG = Logger.getLogger(ProvTlsClient.class.getName());
 
+    private static final boolean provEnableSNIExtension = PropertyUtils.getBooleanSystemProperty("jsse.enableSNIExtension", true);
+
     protected final ProvTlsManager manager;
     protected final ProvSSLParameters sslParameters;
 
+    protected ProvSSLSessionImpl sslSession = null;
     protected boolean handshakeComplete = false;
 
     ProvTlsClient(ProvTlsManager manager)
     {
-        super(manager.getContextData().getCrypto());
+        super(manager.getContextData().getCrypto(), new DefaultTlsKeyExchangeFactory(), new ProvDHConfigVerifier());
 
         this.manager = manager;
         this.sslParameters = manager.getProvSSLParameters();
     }
 
+    @Override
+    protected CertificateStatusRequest getCertificateStatusRequest()
+    {
+        return null;
+    }
+
+    @Override
+    protected Vector getSupportedGroups(boolean offeringDH, boolean offeringEC)
+    {
+        return SupportedGroups.getClientSupportedGroups(manager.getContext().isFips(), offeringDH, offeringEC);
+    }
+
+    @Override
+    protected Vector getSNIServerNames()
+    {
+        if (provEnableSNIExtension)
+        {
+            List<BCSNIServerName> sniServerNames = manager.getProvSSLParameters().getServerNames();
+            if (sniServerNames == null)
+            {
+                String peerHost = manager.getPeerHost();
+                if (peerHost != null && peerHost.indexOf('.') > 0 && !IPAddress.isValid(peerHost))
+                {
+                    Vector serverNames = new Vector(1);
+                    serverNames.addElement(new ServerName(NameType.host_name, peerHost));
+                    return serverNames;
+                }
+            }
+            else
+            {
+                Vector serverNames = new Vector(sniServerNames.size());
+                for (BCSNIServerName sniServerName : sniServerNames)
+                {
+                    /*
+                     * TODO[jsse] Add support for constructing ServerName using
+                     * BCSNIServerName.getEncoded() directly, then remove the 'host_name' limitation
+                     * (although it's currently the only defined type).
+                     */
+                    if (sniServerName.getType() == NameType.host_name)
+                    {
+                        try
+                        {
+                            serverNames.addElement(new ServerName((short)sniServerName.getType(), new String(sniServerName.getEncoded(), "ASCII")));
+                        }
+                        catch (UnsupportedEncodingException e)
+                        {
+                            LOG.log(Level.WARNING, "Unable to include SNI server name", e);
+                        }
+                    }
+                }
+
+                // NOTE: We follow SunJSSE behaviour and disable SNI if there are no server names to send
+                if (!serverNames.isEmpty())
+                {
+                    return serverNames;
+                }
+            }
+        }
+        return null;
+    }
+
     @Override
     protected Vector getSupportedSignatureAlgorithms()
     {
@@ -193,16 +265,17 @@ class ProvTlsClient
                 }
             }
 
-            public void notifyServerCertificate(Certificate serverCertificate) throws IOException
+            public void notifyServerCertificate(TlsServerCertificate serverCertificate) throws IOException
             {
-                boolean noServerCert = serverCertificate == null || serverCertificate.isEmpty();
+                boolean noServerCert = serverCertificate == null || serverCertificate.getCertificate() == null
+                    || serverCertificate.getCertificate().isEmpty();
                 if (noServerCert)
                 {
                     throw new TlsFatalAlert(AlertDescription.handshake_failure);
                 }
                 else
                 {
-                    X509Certificate[] chain = JsseUtils.getX509CertificateChain(serverCertificate);
+                    X509Certificate[] chain = JsseUtils.getX509CertificateChain(manager.getContextData().getCrypto(), serverCertificate.getCertificate());
                     String authType = JsseUtils.getAuthTypeServer(TlsUtils.getKeyExchangeAlgorithm(selectedCipherSuite));
 
                     if (!manager.isServerTrusted(chain, authType))
@@ -222,23 +295,11 @@ class ProvTlsClient
     }
 
     @Override
-    public Hashtable getClientExtensions() throws IOException
+    public short[] getCompressionMethods()
     {
-        Hashtable clientExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(super.getClientExtensions());
-
-        boolean enableSNIExtension = !"false".equals(PropertyUtils.getSystemProperty("jsse.enableSNIExtension"));
-        if (enableSNIExtension)
-        {
-            String peerHost = manager.getPeerHost();
-            if (peerHost != null && peerHost.indexOf('.') > 0 && !IPAddress.isValid(peerHost))
-            {
-                Vector serverNames = new Vector(1);
-                serverNames.addElement(new ServerName(NameType.host_name, peerHost));
-                TlsExtensionsUtils.addServerNameExtension(clientExtensions, new ServerNameList(serverNames));
-            }
-        }
-
-        return clientExtensions;
+        return manager.getContext().isFips()
+            ?   new short[]{ CompressionMethod._null }
+            :   super.getCompressionMethods();
     }
 
 //    public TlsKeyExchange getKeyExchange() throws IOException
@@ -262,7 +323,23 @@ class ProvTlsClient
     @Override
     public TlsSession getSessionToResume()
     {
-        // TODO[jsse] Search for a suitable session in the client session context
+        ProvSSLSessionContext sessionContext = manager.getContextData().getClientSessionContext();
+        this.sslSession = sessionContext.getSessionImpl(manager.getPeerHost(), manager.getPeerPort());
+
+        if (sslSession != null)
+        {
+            TlsSession sessionToResume = sslSession.getTlsSession();
+            if (sessionToResume != null)
+            {
+                return sessionToResume;
+            }
+        }
+
+        if (!manager.getEnableSessionCreation())
+        {
+            throw new IllegalStateException("No resumable sessions and session creation is disabled");
+        }
+
         return null;
     }
 
@@ -308,16 +385,39 @@ class ProvTlsClient
     {
         this.handshakeComplete = true;
 
-        ProvSSLSessionContext sessionContext = manager.getContextData().getClientSessionContext();
-        SSLSession session = sessionContext.reportSession(context.getSession());
-        ProvSSLConnection connection = new ProvSSLConnection(context, session);
+        TlsSession handshakeSession = context.getSession();
 
-        manager.notifyHandshakeComplete(connection);
+        if (sslSession == null || sslSession.getTlsSession() != handshakeSession)
+        {
+            sslSession = manager.getContextData().getClientSessionContext().reportSession(handshakeSession,
+                manager.getPeerHost(), manager.getPeerPort());
+        }
+
+        manager.notifyHandshakeComplete(new ProvSSLConnection(context, sslSession));
+    }
+
+    @Override
+    public void notifySecureRenegotiation(boolean secureRenegotiation) throws IOException
+    {
+        if (!secureRenegotiation)
+        {
+            boolean allowLegacyHelloMessages = PropertyUtils.getBooleanSystemProperty("sun.security.ssl.allowLegacyHelloMessages", true);
+            if (!allowLegacyHelloMessages)
+            {
+                /*
+                 * RFC 5746 3.4/3.6. In this case, some clients/servers may want to terminate the handshake instead
+                 * of continuing; see Section 4.1/4.3 for discussion.
+                 */
+                throw new TlsFatalAlert(AlertDescription.handshake_failure);
+            }
+        }
     }
 
     @Override
     public void notifySelectedCipherSuite(int selectedCipherSuite)
     {
+        manager.getContext().validateNegotiatedCipherSuite(selectedCipherSuite);
+
         super.notifySelectedCipherSuite(selectedCipherSuite);
 
         LOG.fine("Client notified of selected cipher suite: " + manager.getContext().getCipherSuiteString(selectedCipherSuite));
@@ -340,4 +440,27 @@ class ProvTlsClient
         }
         throw new TlsFatalAlert(AlertDescription.protocol_version);
     }
+
+    @Override
+    public void notifySessionID(byte[] sessionID)
+    {
+        super.notifySessionID(sessionID);
+
+        if (sessionID == null || sessionID.length == 0)
+        {
+            LOG.fine("Server did not specify a session ID");
+        }
+        else if (sslSession != null && Arrays.areEqual(sessionID, sslSession.getId()))
+        {
+            LOG.fine("Server resumed session: " + Hex.toHexString(sessionID));
+        }
+        else if (!manager.getEnableSessionCreation())
+        {
+            throw new IllegalStateException("Server did not resume session and session creation is disabled");
+        }
+        else
+        {
+            LOG.fine("Server specified new session: " + Hex.toHexString(sessionID));
+        }
+    }
 }
