@@ -19,8 +19,8 @@ public abstract class AbstractTlsClient
 
     protected TlsClientContext context;
 
+    protected Vector supportedGroups;
     protected Vector supportedSignatureAlgorithms;
-    protected int[] namedCurves;
     protected short[] clientECPointFormats, serverECPointFormats;
 
     protected int selectedCipherSuite;
@@ -49,8 +49,18 @@ public abstract class AbstractTlsClient
              * Supported Elliptic Curves Extension is clearly intended to be client-only. If
              * present, we still require that it is a valid EllipticCurveList.
              */
-            TlsECCUtils.readSupportedEllipticCurvesExtension(extensionData);
+            TlsExtensionsUtils.readSupportedGroupsExtension(extensionData);
             return true;
+
+        case ExtensionType.ec_point_formats:
+            /*
+             * Exception added based on field reports that some servers send this even when they
+             * didn't negotiate an ECC cipher suite. If present, we still require that it is a valid
+             * ECPointFormatList.
+             */
+            TlsECCUtils.readSupportedPointFormatsExtension(extensionData);
+            return true;
+
         default:
             return false;
         }
@@ -69,7 +79,59 @@ public abstract class AbstractTlsClient
     protected TlsECConfigVerifier createECConfigVerifier()
     {
         int minimumCurveBits = TlsECCUtils.getMinimumCurveBits(selectedCipherSuite);
-        return new DefaultTlsECConfigVerifier(minimumCurveBits, namedCurves);
+        return new DefaultTlsECConfigVerifier(minimumCurveBits, supportedGroups);
+    }
+
+    protected CertificateStatusRequest getCertificateStatusRequest()
+    {
+        return new CertificateStatusRequest(CertificateStatusType.ocsp, new OCSPStatusRequest(null, null));
+    }
+
+    protected Vector getSNIServerNames()
+    {
+        return null;
+    }
+
+    protected short[] getSupportedPointFormats()
+    {
+        return new short[]{ ECPointFormat.uncompressed, ECPointFormat.ansiX962_compressed_prime,
+            ECPointFormat.ansiX962_compressed_char2, };
+    }
+
+    /**
+     * The default {@link #getClientExtensions()} implementation calls this to determine which named
+     * groups to include in the supported_groups extension for the ClientHello.
+     * 
+     * @param offeringDH
+     *            True if we are offering any DH ciphersuites in ClientHello, so at least one DH
+     *            group should be included.
+     * @param offeringEC
+     *            True if we are offering any EC ciphersuites in ClientHello, so at least one EC
+     *            group should be included.
+     * @return a {@link Vector} of {@link Integer}. See {@link NamedGroup} for group constants.
+     */
+    protected Vector getSupportedGroups(boolean offeringDH, boolean offeringEC)
+    {
+        Vector supportedGroups = new Vector();
+
+        if (offeringEC)
+        {
+            /*
+             * NOTE[fips]: These curves are recommended for FIPS. If any changes are made to how
+             * this is configured, FIPS considerations need to be accounted for in BCJSSE.
+             */
+            supportedGroups.addElement(NamedGroup.secp256r1);
+            supportedGroups.addElement(NamedGroup.secp384r1);
+        }
+
+        if (offeringDH)
+        {
+            supportedGroups.addElement(NamedGroup.ffdhe2048);
+            supportedGroups.addElement(NamedGroup.ffdhe3072);
+            supportedGroups.addElement(NamedGroup.ffdhe4096);
+        }
+
+        return supportedGroups;
     }
 
     protected Vector getSupportedSignatureAlgorithms()
@@ -117,7 +179,22 @@ public abstract class AbstractTlsClient
     public Hashtable getClientExtensions()
         throws IOException
     {
-        Hashtable clientExtensions = null;
+        Hashtable clientExtensions = new Hashtable();
+
+        TlsExtensionsUtils.addEncryptThenMACExtension(clientExtensions);
+        TlsExtensionsUtils.addExtendedMasterSecretExtension(clientExtensions);
+
+        Vector sniServerNames = getSNIServerNames();
+        if (sniServerNames != null)
+        {
+            TlsExtensionsUtils.addServerNameExtension(clientExtensions, new ServerNameList(sniServerNames));
+        }
+
+        CertificateStatusRequest statusRequest = getCertificateStatusRequest();
+        if (statusRequest != null)
+        {
+            TlsExtensionsUtils.addStatusRequestExtension(clientExtensions, statusRequest);
+        }
 
         ProtocolVersion clientVersion = context.getClientVersion();
 
@@ -129,33 +206,28 @@ public abstract class AbstractTlsClient
         {
             this.supportedSignatureAlgorithms = getSupportedSignatureAlgorithms();
 
-            clientExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(clientExtensions);
-
             TlsUtils.addSignatureAlgorithmsExtension(clientExtensions, supportedSignatureAlgorithms);
         }
 
-        if (TlsECCUtils.containsECCipherSuites(getCipherSuites()))
+        int[] cipherSuites = getCipherSuites();
+        boolean offeringDH = TlsDHUtils.containsDHECipherSuites(cipherSuites);
+        boolean offeringEC = TlsECCUtils.containsECCipherSuites(cipherSuites);
+
+        if (offeringEC)
         {
-            /*
-             * RFC 4492 5.1. A client that proposes ECC cipher suites in its ClientHello message
-             * appends these extensions (along with any others), enumerating the curves it supports
-             * and the point formats it can parse. Clients SHOULD send both the Supported Elliptic
-             * Curves Extension and the Supported Point Formats Extension.
-             */
-            /*
-             * TODO Could just add all the curves since we support them all, but users may not want
-             * to use unnecessarily large fields. Need configuration options.
-             */
-            this.namedCurves = new int[]{ NamedCurve.secp256r1, NamedCurve.secp384r1 };
-            this.clientECPointFormats = new short[]{ ECPointFormat.uncompressed,
-                ECPointFormat.ansiX962_compressed_prime, ECPointFormat.ansiX962_compressed_char2, };
+            this.clientECPointFormats = getSupportedPointFormats();
 
-            clientExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(clientExtensions);
-
-            TlsECCUtils.addSupportedEllipticCurvesExtension(clientExtensions, namedCurves);
             TlsECCUtils.addSupportedPointFormatsExtension(clientExtensions, clientECPointFormats);
         }
 
+        Vector supportedGroups = getSupportedGroups(offeringDH, offeringEC);
+        if (supportedGroups != null && !supportedGroups.isEmpty())
+        {
+            this.supportedGroups = supportedGroups;
+
+            TlsExtensionsUtils.addSupportedGroupsExtension(clientExtensions, supportedGroups);
+        }
+
         return clientExtensions;
     }
 
@@ -207,7 +279,7 @@ public abstract class AbstractTlsClient
              */
             checkForUnexpectedServerExtension(serverExtensions, TlsUtils.EXT_signature_algorithms);
 
-            checkForUnexpectedServerExtension(serverExtensions, TlsECCUtils.EXT_elliptic_curves);
+            checkForUnexpectedServerExtension(serverExtensions, TlsExtensionsUtils.EXT_supported_groups);
 
             if (TlsECCUtils.isECCipherSuite(this.selectedCipherSuite))
             {
