@@ -162,9 +162,18 @@ public abstract class AbstractTlsKeyExchange
         }
     }
 
+    public short[] getClientCertificateTypes()
+    {
+        return null;
+    }
+
     public void skipClientCredentials()
         throws IOException
     {
+        if (TlsUtils.isStaticKeyAgreement(keyExchange))
+        {
+            throw new TlsFatalAlert(AlertDescription.unexpected_message);
+        }
     }
 
     public void processClientCertificate(Certificate clientCertificate)
@@ -178,4 +187,9 @@ public abstract class AbstractTlsKeyExchange
         // Key exchange implementation MUST support client key exchange
         throw new TlsFatalAlert(AlertDescription.internal_error);
     }
+
+    public boolean requiresCertificateVerify()
+    {
+        return !TlsUtils.isStaticKeyAgreement(keyExchange);
+    }
 }
