@@ -26,11 +26,6 @@ public class ByteQueue
         return i + 1;
     }
 
-    /**
-     * The initial size for our buffer.
-     */
-    private static final int DEFAULT_CAPACITY = 1024;
-
     /**
      * The buffer where we store our data.
      */
@@ -50,7 +45,7 @@ public class ByteQueue
 
     public ByteQueue()
     {
-        this(DEFAULT_CAPACITY);
+        this(0);
     }
 
     public ByteQueue(int capacity)
