@@ -3,7 +3,6 @@ package org.bouncycastle.tls;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
-import java.security.SecureRandom;
 import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Vector;
@@ -14,9 +13,9 @@ import org.bouncycastle.util.Arrays;
 public class DTLSClientProtocol
     extends DTLSProtocol
 {
-    public DTLSClientProtocol(SecureRandom secureRandom)
+    public DTLSClientProtocol()
     {
-        super(secureRandom);
+        super();
     }
 
     public DTLSTransport connect(TlsClient client, DatagramTransport transport)
@@ -198,7 +197,7 @@ public class DTLSClientProtocol
         else
         {
             // Okay, Certificate is optional
-            state.keyExchange.skipServerCredentials();
+            state.authentication = null;
         }
 
         // TODO[RFC 3546] Check whether empty certificates is possible, allowed, or excludes CertificateStatus
@@ -217,6 +216,17 @@ public class DTLSClientProtocol
             // Okay, CertificateStatus is optional
         }
 
+        if (state.authentication == null)
+        {
+            // There was no server certificate message; check it's OK
+            state.keyExchange.skipServerCredentials();
+        }
+        else
+        {
+            TlsUtils.processServerCertificate(serverCertificate, state.certificateStatus, state.keyExchange,
+                state.authentication, state.clientExtensions, state.serverExtensions);
+        }
+
         if (serverMessage.getType() == HandshakeType.server_key_exchange)
         {
             processServerKeyExchange(state, serverMessage.getBody());
@@ -518,7 +528,7 @@ public class DTLSClientProtocol
 
         TlsProtocol.assertEmpty(buf);
 
-        state.keyExchange.validateCertificateRequest(state.certificateRequest);
+        state.certificateRequest = TlsUtils.validateCertificateRequest(state.certificateRequest, state.keyExchange);
     }
 
     protected void processCertificateStatus(ClientHandshakeState state, byte[] body)
@@ -539,8 +549,6 @@ public class DTLSClientProtocol
         state.certificateStatus = CertificateStatus.parse(buf);
 
         TlsProtocol.assertEmpty(buf);
-
-        // TODO[RFC 3546] Figure out how to provide this to the client/authentication.
     }
 
     protected byte[] processHelloVerifyRequest(ClientHandshakeState state, byte[] body)
@@ -593,9 +601,11 @@ public class DTLSClientProtocol
 
         TlsProtocol.assertEmpty(buf);
 
-        state.keyExchange.processServerCertificate(serverCertificate);
         state.authentication = state.client.getAuthentication();
-        state.authentication.notifyServerCertificate(serverCertificate);
+        if (state.authentication == null)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
 
         return serverCertificate;
     }
@@ -607,10 +617,8 @@ public class DTLSClientProtocol
 
         ByteArrayInputStream buf = new ByteArrayInputStream(body);
 
-        {
-            ProtocolVersion server_version = TlsUtils.readVersion(buf);
-            reportServerVersion(state, server_version);
-        }
+        ProtocolVersion server_version = TlsUtils.readVersion(buf);
+        reportServerVersion(state, server_version);
 
         securityParameters.serverRandom = TlsUtils.readFully(32, buf);
 
@@ -744,7 +752,8 @@ public class DTLSClientProtocol
         if (state.resumedSession)
         {
             if (selectedCipherSuite != state.sessionParameters.getCipherSuite()
-                || selectedCompressionMethod != state.sessionParameters.getCompressionAlgorithm())
+                || selectedCompressionMethod != state.sessionParameters.getCompressionAlgorithm()
+                || !server_version.equals(state.sessionParameters.getNegotiatedVersion()))
             {
                 throw new TlsFatalAlert(AlertDescription.illegal_parameter);
             }
