@@ -52,6 +52,11 @@ class DTLSRecordLayer
         this.plaintextLimit = plaintextLimit;
     }
 
+    int getReadEpoch()
+    {
+        return readEpoch.getEpoch();
+    }
+
     ProtocolVersion getReadVersion()
     {
         return readVersion;
