@@ -28,14 +28,12 @@ public class TlsClientProtocol
     /**
      * Constructor for non-blocking mode.<br>
      * <br>
-     * When data is received, use {@link #offerInput(java.nio.ByteBuffer)} to
-     * provide the received ciphertext, then use
-     * {@link #readInput(byte[], int, int)} to read the corresponding cleartext.<br>
+     * When data is received, use {@link #offerInput(byte[])} to provide the received ciphertext,
+     * then use {@link #readInput(byte[], int, int)} to read the corresponding cleartext.<br>
      * <br>
-     * Similarly, when data needs to be sent, use
-     * {@link #offerOutput(byte[], int, int)} to provide the cleartext, then use
-     * {@link #readOutput(byte[], int, int)} to get the corresponding
-     * ciphertext.
+     * Similarly, when data needs to be sent, use {@link #writeApplicationData(byte[], int, int)} to
+     * provide the cleartext, then use {@link #readOutput(byte[], int, int)} to get the
+     * corresponding ciphertext.
      */
     public TlsClientProtocol()
     {
@@ -141,6 +139,7 @@ public class TlsClientProtocol
             processFinishedMessage(buf);
             this.connection_state = CS_SERVER_FINISHED;
 
+            sendChangeCipherSpecMessage();
             sendFinishedMessage();
             this.connection_state = CS_CLIENT_FINISHED;
 
@@ -173,10 +172,11 @@ public class TlsClientProtocol
                     this.allowCertificateStatus = false;
                 }
 
-                this.keyExchange.processServerCertificate(this.peerCertificate);
-
                 this.authentication = tlsClient.getAuthentication();
-                this.authentication.notifyServerCertificate(this.peerCertificate);
+                if (this.authentication == null)
+                {
+                    throw new TlsFatalAlert(AlertDescription.internal_error);
+                }
 
                 break;
             }
@@ -207,8 +207,6 @@ public class TlsClientProtocol
 
                 assertEmpty(buf);
 
-                // TODO[RFC 3546] Figure out how to provide this to the client/authentication.
-
                 this.connection_state = CS_CERTIFICATE_STATUS;
                 break;
             }
@@ -264,8 +262,6 @@ public class TlsClientProtocol
                 {
                     this.securityParameters.masterSecret = getContext().getCrypto().adoptSecret(sessionParameters.getMasterSecret());
                     this.recordStream.setPendingConnectionState(getPeer().getCompression(), getPeer().getCipher());
-
-                    sendChangeCipherSpecMessage();
                 }
                 else
                 {
@@ -307,8 +303,6 @@ public class TlsClientProtocol
             }
             case CS_SERVER_SUPPLEMENTAL_DATA:
             {
-                // There was no server certificate message; check it's OK
-                this.keyExchange.skipServerCredentials();
                 this.authentication = null;
 
                 // NB: Fall through to next case label
@@ -316,6 +310,8 @@ public class TlsClientProtocol
             case CS_SERVER_CERTIFICATE:
             case CS_CERTIFICATE_STATUS:
             {
+                handleServerCertificate();
+
                 // There was no server key exchange message; check it's OK
                 this.keyExchange.skipServerKeyExchange();
 
@@ -385,18 +381,10 @@ public class TlsClientProtocol
                 sendClientKeyExchangeMessage();
                 this.connection_state = CS_CLIENT_KEY_EXCHANGE;
 
-                if (TlsUtils.isSSL(getContext()))
-                {
-                    establishMasterSecret(getContext(), keyExchange);
-                }
-
                 TlsHandshakeHash prepareFinishHash = recordStream.prepareToFinish();
                 this.securityParameters.sessionHash = TlsUtils.getCurrentPRFHash(prepareFinishHash);
 
-                if (!TlsUtils.isSSL(getContext()))
-                {
-                    establishMasterSecret(getContext(), keyExchange);
-                }
+                establishMasterSecret(getContext(), keyExchange);
 
                 recordStream.setPendingConnectionState(getPeer().getCompression(), getPeer().getCipher());
 
@@ -430,8 +418,6 @@ public class TlsClientProtocol
             }
             case CS_SERVER_SUPPLEMENTAL_DATA:
             {
-                // There was no server certificate message; check it's OK
-                this.keyExchange.skipServerCredentials();
                 this.authentication = null;
 
                 // NB: Fall through to next case label
@@ -439,6 +425,8 @@ public class TlsClientProtocol
             case CS_SERVER_CERTIFICATE:
             case CS_CERTIFICATE_STATUS:
             {
+                handleServerCertificate();
+
                 this.keyExchange.processServerKeyExchange(buf);
 
                 assertEmpty(buf);
@@ -458,6 +446,8 @@ public class TlsClientProtocol
             case CS_SERVER_CERTIFICATE:
             case CS_CERTIFICATE_STATUS:
             {
+                handleServerCertificate();
+
                 // There was no server key exchange message; check it's OK
                 this.keyExchange.skipServerKeyExchange();
 
@@ -478,7 +468,7 @@ public class TlsClientProtocol
 
                 assertEmpty(buf);
 
-                this.keyExchange.validateCertificateRequest(this.certificateRequest);
+                this.certificateRequest = TlsUtils.validateCertificateRequest(this.certificateRequest, this.keyExchange);
 
                 /*
                  * TODO Give the client a chance to immediately select the CertificateVerify hash
@@ -552,6 +542,21 @@ public class TlsClientProtocol
         }
     }
 
+    protected void handleServerCertificate()
+        throws IOException
+    {
+        if (this.authentication == null)
+        {
+            // There was no server certificate message; check it's OK
+            this.keyExchange.skipServerCredentials();
+        }
+        else
+        {
+            TlsUtils.processServerCertificate(peerCertificate, certificateStatus, keyExchange, authentication,
+                clientExtensions, serverExtensions);
+        }
+    }
+
     protected void handleSupplementalData(Vector serverSupplementalData)
         throws IOException
     {
@@ -575,9 +580,10 @@ public class TlsClientProtocol
     protected void receiveServerHelloMessage(ByteArrayInputStream buf)
         throws IOException
     {
+        ProtocolVersion server_version = TlsUtils.readVersion(buf);
+
         {
-            ProtocolVersion server_version = TlsUtils.readVersion(buf);
-            if (server_version.isDTLS())
+            if (!TlsUtils.isTLSv10(server_version))
             {
                 throw new TlsFatalAlert(AlertDescription.illegal_parameter);
             }
@@ -734,7 +740,8 @@ public class TlsClientProtocol
         if (this.resumedSession)
         {
             if (selectedCipherSuite != this.sessionParameters.getCipherSuite()
-                || selectedCompressionMethod != this.sessionParameters.getCompressionAlgorithm())
+                || selectedCompressionMethod != this.sessionParameters.getCompressionAlgorithm()
+                || !server_version.equals(this.sessionParameters.getNegotiatedVersion()))
             {
                 throw new TlsFatalAlert(AlertDescription.illegal_parameter);
             }
@@ -822,7 +829,7 @@ public class TlsClientProtocol
         this.recordStream.setWriteVersion(this.tlsClient.getClientHelloRecordLayerVersion());
 
         ProtocolVersion client_version = this.tlsClient.getClientVersion();
-        if (client_version.isDTLS())
+        if (!TlsUtils.isTLSv10(client_version))
         {
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
