@@ -18,20 +18,17 @@ public class TlsServerProtocol
     protected TlsCredentials serverCredentials = null;
     protected CertificateRequest certificateRequest = null;
 
-    protected short clientCertificateType = -1;
     protected TlsHandshakeHash prepareFinishHash = null;
 
     /**
      * Constructor for non-blocking mode.<br>
      * <br>
-     * When data is received, use {@link #offerOutput(byte[], int, int)} to
-     * provide the received ciphertext, then use
-     * {@link #readInput(byte[], int, int)} to read the corresponding cleartext.<br>
+     * When data is received, use {@link #offerInput(byte[])} to provide the received ciphertext,
+     * then use {@link #readInput(byte[], int, int)} to read the corresponding cleartext.<br>
      * <br>
-     * Similarly, when data needs to be sent, use
-     * {@link #offerOutput(byte[], int, int)} to provide the cleartext, then use
-     * {@link #readOutput(byte[], int, int)} to get the corresponding
-     * ciphertext.                                                           fetch
+     * Similarly, when data needs to be sent, use {@link #writeApplicationData(byte[], int, int)} to
+     * provide the cleartext, then use {@link #readOutput(byte[], int, int)} to get the
+     * corresponding ciphertext.
      */
     public TlsServerProtocol()
     {
@@ -200,7 +197,7 @@ public class TlsServerProtocol
                             throw new TlsFatalAlert(AlertDescription.internal_error);
                         }
 
-                        this.keyExchange.validateCertificateRequest(certificateRequest);
+                        this.certificateRequest = TlsUtils.validateCertificateRequest(this.certificateRequest, this.keyExchange);
 
                         sendCertificateRequestMessage(certificateRequest);
 
@@ -294,13 +291,6 @@ public class TlsServerProtocol
                          */
                         throw new TlsFatalAlert(AlertDescription.unexpected_message);
                     }
-                    else if (TlsUtils.isSSL(getContext()))
-                    {
-                        if (this.peerCertificate == null)
-                        {
-                            throw new TlsFatalAlert(AlertDescription.unexpected_message);
-                        }
-                    }
                     else
                     {
                         notifyClientCertificate(Certificate.EMPTY_CHAIN);
@@ -365,10 +355,10 @@ public class TlsServerProtocol
                 if (this.expectSessionTicket)
                 {
                     sendNewSessionTicketMessage(tlsServer.getNewSessionTicket());
-                    sendChangeCipherSpecMessage();
                 }
                 this.connection_state = CS_SERVER_SESSION_TICKET;
 
+                sendChangeCipherSpecMessage();
                 sendFinishedMessage();
                 this.connection_state = CS_SERVER_FINISHED;
 
@@ -392,27 +382,16 @@ public class TlsServerProtocol
         }
     }
 
-    protected void handleWarningMessage(short description)
+    protected void handleAlertWarningMessage(short alertDescription)
         throws IOException
     {
-        switch (description)
+        super.handleAlertWarningMessage(alertDescription);
+
+        switch (alertDescription)
         {
         case AlertDescription.no_certificate:
         {
-            /*
-             * SSL 3.0 If the server has sent a certificate request Message, the client must send
-             * either the certificate message or a no_certificate alert.
-             */
-            if (TlsUtils.isSSL(getContext()) && certificateRequest != null)
-            {
-                notifyClientCertificate(Certificate.EMPTY_CHAIN);
-            }
-            break;
-        }
-        default:
-        {
-            super.handleWarningMessage(description);
-            break;
+            throw new TlsFatalAlert(AlertDescription.unexpected_message);
         }
         }
     }
@@ -445,8 +424,6 @@ public class TlsServerProtocol
              * issued by one of the listed CAs.
              */
 
-            this.clientCertificateType = clientCertificate.getCertificateAt(0).getClientCertificateType();
-
             this.keyExchange.processClientCertificate(clientCertificate);
         }
 
@@ -484,8 +461,8 @@ public class TlsServerProtocol
 
         assertEmpty(buf);
 
-        TlsUtils.verifyCertificateVerify(context, certificateRequest, peerCertificate, clientCertificateType,
-            clientCertificateVerify, prepareFinishHash);
+        TlsUtils.verifyCertificateVerify(context, certificateRequest, peerCertificate, clientCertificateVerify,
+            prepareFinishHash);
     }
 
     protected void receiveClientHelloMessage(ByteArrayInputStream buf)
@@ -618,25 +595,12 @@ public class TlsServerProtocol
 
         assertEmpty(buf);
 
-        if (TlsUtils.isSSL(getContext()))
-        {
-            establishMasterSecret(getContext(), keyExchange);
-        }
-
         this.prepareFinishHash = recordStream.prepareToFinish();
         this.securityParameters.sessionHash = TlsUtils.getCurrentPRFHash(prepareFinishHash);
 
-        if (!TlsUtils.isSSL(getContext()))
-        {
-            establishMasterSecret(getContext(), keyExchange);
-        }
+        establishMasterSecret(getContext(), keyExchange);
 
         recordStream.setPendingConnectionState(getPeer().getCompression(), getPeer().getCipher());
-
-        if (!expectSessionTicket)
-        {
-            sendChangeCipherSpecMessage();
-        }
     }
 
     protected void sendCertificateRequestMessage(CertificateRequest certificateRequest)
@@ -822,6 +786,6 @@ public class TlsServerProtocol
 
     protected boolean expectCertificateVerifyMessage()
     {
-        return clientCertificateType >= 0 && TlsUtils.hasSigningCapability(clientCertificateType);
+        return peerCertificate != null && !peerCertificate.isEmpty() && keyExchange.requiresCertificateVerify();
     }
 }
