@@ -1,7 +1,9 @@
 package org.bouncycastle.tls.crypto;
 
 import java.io.IOException;
+import java.math.BigInteger;
 
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.tls.ClientCertificateType;
 import org.bouncycastle.tls.ConnectionEnd;
 import org.bouncycastle.tls.KeyExchangeAlgorithm;
@@ -25,6 +27,10 @@ public interface TlsCertificate
 
     byte[] getEncoded() throws IOException;
 
+    byte[] getExtension(ASN1ObjectIdentifier extensionOID) throws IOException;
+
+    BigInteger getSerialNumber();
+
     /**
      * @param connectionEnd
      *            {@link ConnectionEnd}
