@@ -17,6 +17,7 @@ import org.bouncycastle.crypto.agreement.srp.SRP6Client;
 import org.bouncycastle.crypto.agreement.srp.SRP6Server;
 import org.bouncycastle.crypto.agreement.srp.SRP6VerifierGenerator;
 import org.bouncycastle.crypto.digests.MD5Digest;
+import org.bouncycastle.crypto.digests.NullDigest;
 import org.bouncycastle.crypto.digests.SHA1Digest;
 import org.bouncycastle.crypto.digests.SHA224Digest;
 import org.bouncycastle.crypto.digests.SHA256Digest;
@@ -26,13 +27,11 @@ import org.bouncycastle.crypto.encodings.PKCS1Encoding;
 import org.bouncycastle.crypto.engines.AESEngine;
 import org.bouncycastle.crypto.engines.ARIAEngine;
 import org.bouncycastle.crypto.engines.CamelliaEngine;
-import org.bouncycastle.crypto.engines.ChaCha7539Engine;
 import org.bouncycastle.crypto.engines.DESedeEngine;
 import org.bouncycastle.crypto.engines.RC4Engine;
 import org.bouncycastle.crypto.engines.RSABlindedEngine;
 import org.bouncycastle.crypto.engines.SEEDEngine;
 import org.bouncycastle.crypto.macs.HMac;
-import org.bouncycastle.crypto.macs.Poly1305;
 import org.bouncycastle.crypto.modes.AEADBlockCipher;
 import org.bouncycastle.crypto.modes.CBCBlockCipher;
 import org.bouncycastle.crypto.modes.CCMBlockCipher;
@@ -46,11 +45,9 @@ import org.bouncycastle.crypto.params.RSAKeyParameters;
 import org.bouncycastle.crypto.params.SRP6GroupParameters;
 import org.bouncycastle.crypto.prng.DigestRandomGenerator;
 import org.bouncycastle.tls.AlertDescription;
-import org.bouncycastle.tls.CombinedHash;
 import org.bouncycastle.tls.EncryptionAlgorithm;
 import org.bouncycastle.tls.HashAlgorithm;
-import org.bouncycastle.tls.MACAlgorithm;
-import org.bouncycastle.tls.PRFAlgorithm;
+import org.bouncycastle.tls.NamedGroup;
 import org.bouncycastle.tls.ProtocolVersion;
 import org.bouncycastle.tls.SignatureAndHashAlgorithm;
 import org.bouncycastle.tls.TlsFatalAlert;
@@ -65,24 +62,20 @@ import org.bouncycastle.tls.crypto.TlsECDomain;
 import org.bouncycastle.tls.crypto.TlsHMAC;
 import org.bouncycastle.tls.crypto.TlsHash;
 import org.bouncycastle.tls.crypto.TlsMAC;
+import org.bouncycastle.tls.crypto.TlsNonceGenerator;
 import org.bouncycastle.tls.crypto.TlsSRP6Client;
 import org.bouncycastle.tls.crypto.TlsSRP6Server;
 import org.bouncycastle.tls.crypto.TlsSRP6VerifierGenerator;
 import org.bouncycastle.tls.crypto.TlsSRPConfig;
 import org.bouncycastle.tls.crypto.TlsSecret;
 import org.bouncycastle.tls.crypto.impl.AbstractTlsCrypto;
-import org.bouncycastle.tls.crypto.impl.ChaCha20Poly1305Cipher;
 import org.bouncycastle.tls.crypto.impl.TlsAEADCipher;
 import org.bouncycastle.tls.crypto.impl.TlsAEADCipherImpl;
 import org.bouncycastle.tls.crypto.impl.TlsBlockCipher;
 import org.bouncycastle.tls.crypto.impl.TlsBlockCipherImpl;
 import org.bouncycastle.tls.crypto.impl.TlsEncryptor;
-import org.bouncycastle.tls.crypto.impl.TlsImplUtils;
 import org.bouncycastle.tls.crypto.impl.TlsNullCipher;
-import org.bouncycastle.tls.crypto.impl.TlsStreamCipher;
-import org.bouncycastle.tls.crypto.impl.TlsStreamCipherImpl;
 import org.bouncycastle.util.Arrays;
-import org.bouncycastle.util.Times;
 
 /**
  * Class for providing cryptographic services for TLS based on implementations in the BC light-weight API.
@@ -94,24 +87,11 @@ import org.bouncycastle.util.Times;
 public class BcTlsCrypto
     extends AbstractTlsCrypto
 {
-    private final DigestRandomGenerator nonceGen;
     private final SecureRandom entropySource;
 
     public BcTlsCrypto(SecureRandom entropySource)
     {
         this.entropySource = entropySource;
-
-        Digest digest = createDigest(HashAlgorithm.sha256);
-
-        nonceGen = new DigestRandomGenerator(digest);
-
-        nonceGen.addSeedMaterial(nextCounterValue());
-        nonceGen.addSeedMaterial(Times.nanoTime());
-
-        byte[] seed = new byte[digest.getDigestSize()];
-        entropySource.nextBytes(seed);
-
-        nonceGen.addSeedMaterial(seed);
     }
 
     BcTlsSecret adoptLocalSecret(byte[] data)
@@ -119,15 +99,6 @@ public class BcTlsCrypto
         return new BcTlsSecret(this, data);
     }
 
-    public byte[] createNonce(int size)
-    {
-        byte[] nonce = new byte[size];
-
-        nonceGen.nextBytes(nonce);
-
-        return nonce;
-    }
-
     public SecureRandom getSecureRandom()
     {
         return entropySource;
@@ -199,8 +170,6 @@ public class BcTlsCrypto
             return createChaCha20Poly1305(cryptoParams);
         case EncryptionAlgorithm.NULL:
             return createNullCipher(cryptoParams, macAlgorithm);
-        case EncryptionAlgorithm.RC4_128:
-            return createRC4Cipher(cryptoParams, 16, macAlgorithm);
         case EncryptionAlgorithm.SEED_CBC:
             return createSEEDCipher(cryptoParams, macAlgorithm);
         default:
@@ -248,6 +217,31 @@ public class BcTlsCrypto
         };
     }
 
+    public TlsNonceGenerator createNonceGenerator(byte[] additionalSeedMaterial)
+    {
+        final DigestRandomGenerator nonceGen = new DigestRandomGenerator(createDigest(HashAlgorithm.sha256));
+
+        if (additionalSeedMaterial != null && additionalSeedMaterial.length > 0)
+        {
+            nonceGen.addSeedMaterial(additionalSeedMaterial);
+        }
+
+        byte[] seed = new byte[createDigest(HashAlgorithm.sha256).getDigestSize()];
+        entropySource.nextBytes(seed);
+
+        nonceGen.addSeedMaterial(seed);
+
+        return new TlsNonceGenerator()
+        {
+            public byte[] generateNonce(int size)
+            {
+                byte[] nonce = new byte[size];
+                nonceGen.nextBytes(nonce);
+                return nonce;
+            }
+        };
+    }
+
     public boolean hasAllRawSignatureAlgorithms()
     {
         return true;
@@ -265,7 +259,19 @@ public class BcTlsCrypto
 
     public boolean hasEncryptionAlgorithm(int encryptionAlgorithm)
     {
-        return true;
+        switch (encryptionAlgorithm)
+        {
+        case EncryptionAlgorithm.DES40_CBC:
+        case EncryptionAlgorithm.DES_CBC:
+        case EncryptionAlgorithm.IDEA_CBC:
+        case EncryptionAlgorithm.RC2_CBC_40:
+        case EncryptionAlgorithm.RC4_128:
+        case EncryptionAlgorithm.RC4_40:
+            return false;
+
+        default:
+            return true;
+        }
     }
 
     public boolean hasHashAlgorithm(short hashAlgorithm)
@@ -278,10 +284,9 @@ public class BcTlsCrypto
         return true;
     }
 
-    public boolean hasNamedCurve(int curveID)
+    public boolean hasNamedGroup(int namedGroup)
     {
-        // TODO: at the moment we support everything
-        return true;
+        return NamedGroup.refersToASpecificGroup(namedGroup);
     }
 
     public boolean hasRSAEncryption()
@@ -328,25 +333,12 @@ public class BcTlsCrypto
         return adoptLocalSecret(data);
     }
 
-//    public byte[] calculateKeyBlock(TlsContext context, int length)
-//    {
-//        SecurityParameters securityParameters = context.getSecurityParameters();
-//        byte[] master_secret = securityParameters.getMasterSecret();
-//        byte[] seed = concat(securityParameters.getServerRandom(), securityParameters.getClientRandom());
-//
-//        if (isSSL(context))
-//        {
-//            return context.getCrypto().createSecret(master_secret).deriveSSLKeyBlock(seed, length).extract();
-//        }
-//
-//        return PRF(context, master_secret, ExporterLabel.key_expansion, seed, length);
-//    }
-
-    // TODO[tls-ops] Shouldn't be static, need to pass BcTlsCrypto instance to callers
-    static Digest createDigest(short hashAlgorithm)
+    public Digest createDigest(short hashAlgorithm)
     {
         switch (hashAlgorithm)
         {
+        case HashAlgorithm.none:
+            return new NullDigest();
         case HashAlgorithm.md5:
             return new MD5Digest();
         case HashAlgorithm.sha1:
@@ -360,20 +352,10 @@ public class BcTlsCrypto
         case HashAlgorithm.sha512:
             return new SHA512Digest();
         default:
-            throw new IllegalArgumentException("unknown HashAlgorithm");
+            throw new IllegalArgumentException("unknown HashAlgorithm: " + HashAlgorithm.getText(hashAlgorithm));
         }
     }
 
-    public TlsHash createHash(final SignatureAndHashAlgorithm signatureAndHashAlgorithm)
-    {
-        if (signatureAndHashAlgorithm == null)
-        {
-            return new CombinedHash(this);
-        }
-
-        return new BcTlsHash(signatureAndHashAlgorithm.getHash(), createDigest(signatureAndHashAlgorithm.getHash()));
-    }
-
     public TlsHash createHash(short algorithm)
     {
         return new BcTlsHash(algorithm, createDigest(algorithm));
@@ -399,9 +381,7 @@ public class BcTlsCrypto
         public byte[] calculateHash()
         {
             byte[] rv = new byte[digest.getDigestSize()];
-
             digest.doFinal(rv, 0);
-
             return rv;
         }
 
@@ -433,18 +413,7 @@ public class BcTlsCrypto
         case HashAlgorithm.sha512:
             return new SHA512Digest((SHA512Digest)hash);
         default:
-            throw new IllegalArgumentException("unknown HashAlgorithm");
-        }
-    }
-
-    public Digest createPRFHash(int prfAlgorithm)
-    {
-        switch (prfAlgorithm)
-        {
-        case PRFAlgorithm.tls_prf_legacy:
-            return new CombinedPRF();
-        default:
-            return createDigest(TlsUtils.getHashAlgorithmForPRFAlgorithm(prfAlgorithm));
+            throw new IllegalArgumentException("unknown HashAlgorithm: " + HashAlgorithm.getText(hashAlgorithm));
         }
     }
 
@@ -452,29 +421,27 @@ public class BcTlsCrypto
         throws IOException
     {
         return new TlsBlockCipher(this, cryptoParams, new BlockOperator(createAESBlockCipher(), true), new BlockOperator(createAESBlockCipher(), false),
-            createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), cipherKeySize);
+            createHMAC(macAlgorithm), createHMAC(macAlgorithm), cipherKeySize);
     }
 
     protected TlsCipher createARIACipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
         throws IOException
     {
         return new TlsBlockCipher(this, cryptoParams, new BlockOperator(createARIABlockCipher(), true), new BlockOperator(createARIABlockCipher(), false),
-            createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), cipherKeySize);
+            createHMAC(macAlgorithm), createHMAC(macAlgorithm), cipherKeySize);
     }
 
     protected TlsCipher createCamelliaCipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
         throws IOException
     {
         return new TlsBlockCipher(this, cryptoParams, new BlockOperator(createCamelliaBlockCipher(), true), new BlockOperator(createCamelliaBlockCipher(), false),
-            createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), cipherKeySize);
+            createHMAC(macAlgorithm), createHMAC(macAlgorithm), cipherKeySize);
     }
 
     protected TlsCipher createChaCha20Poly1305(TlsCryptoParameters cryptoParams)
         throws IOException
     {
-        return new ChaCha20Poly1305Cipher(cryptoParams,
-                new StreamOperator(new ChaCha7539Engine(), true), new StreamOperator(new ChaCha7539Engine(), false),
-            new MacOperator(new Poly1305()), new MacOperator(new Poly1305()));
+        return new TlsAEADCipher(cryptoParams, new BcChaCha20Poly1305(true), new BcChaCha20Poly1305(false), 32, 16, TlsAEADCipher.NONCE_RFC7905);
     }
 
     protected TlsAEADCipher createCipher_AES_CCM(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
@@ -516,27 +483,20 @@ public class BcTlsCrypto
         throws IOException
     {
         return new TlsBlockCipher(this, cryptoParams, new BlockOperator(createDESedeBlockCipher(), true), new BlockOperator(createDESedeBlockCipher(), false),
-            createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), 24);
+            createHMAC(macAlgorithm), createHMAC(macAlgorithm), 24);
     }
 
     protected TlsNullCipher createNullCipher(TlsCryptoParameters cryptoParams, int macAlgorithm)
         throws IOException
     {
-        return new TlsNullCipher(cryptoParams, createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm));
-    }
-
-    protected TlsStreamCipher createRC4Cipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
-        throws IOException
-    {
-        return new TlsStreamCipher(cryptoParams, new StreamOperator(createRC4StreamCipher(), true), new StreamOperator(createRC4StreamCipher(), false),
-            createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), cipherKeySize, false);
+        return new TlsNullCipher(cryptoParams, createHMAC(macAlgorithm), createHMAC(macAlgorithm));
     }
 
     protected TlsBlockCipher createSEEDCipher(TlsCryptoParameters cryptoParams, int macAlgorithm)
         throws IOException
     {
         return new TlsBlockCipher(this, cryptoParams, new BlockOperator(createSEEDBlockCipher(), true), new BlockOperator(createSEEDBlockCipher(), false),
-            createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), 16);
+            createHMAC(macAlgorithm), createHMAC(macAlgorithm), 16);
     }
 
     protected BlockCipher createAESEngine()
@@ -612,58 +572,9 @@ public class BcTlsCrypto
         return new CBCBlockCipher(new SEEDEngine());
     }
 
-    private TlsHMAC createMac(TlsCryptoParameters cryptoParams, int macAlgorithm)
-        throws IOException
+    public TlsHMAC createHMAC(int macAlgorithm)
     {
-        if (TlsImplUtils.isSSL(cryptoParams))
-        {
-            return createSSl3HMAC((short)macAlgorithm);
-        }
-        else
-        {
-            return createHMAC((short)macAlgorithm);
-        }
-    }
-
-    protected Digest createHMACDigest(int macAlgorithm)
-        throws IOException
-    {
-        switch (macAlgorithm)
-        {
-        case MACAlgorithm._null:
-            return null;
-        case MACAlgorithm.hmac_md5:
-            return createDigest(HashAlgorithm.md5);
-        case MACAlgorithm.hmac_sha1:
-            return createDigest(HashAlgorithm.sha1);
-        case MACAlgorithm.hmac_sha256:
-            return createDigest(HashAlgorithm.sha256);
-        case MACAlgorithm.hmac_sha384:
-            return createDigest(HashAlgorithm.sha384);
-        case MACAlgorithm.hmac_sha512:
-            return createDigest(HashAlgorithm.sha512);
-        default:
-            throw new TlsFatalAlert(AlertDescription.internal_error);
-        }
-    }
-
-    public TlsHMAC createHMAC(int hashAlgorithm)
-    {
-        switch (hashAlgorithm)
-        {
-        case MACAlgorithm.hmac_md5:
-            return new HMacOperator(createDigest(HashAlgorithm.md5));
-        case MACAlgorithm.hmac_sha1:
-            return new HMacOperator(createDigest(HashAlgorithm.sha1));
-        case MACAlgorithm.hmac_sha256:
-            return new HMacOperator(createDigest(HashAlgorithm.sha256));
-        case MACAlgorithm.hmac_sha384:
-            return new HMacOperator(createDigest(HashAlgorithm.sha384));
-        case MACAlgorithm.hmac_sha512:
-            return new HMacOperator(createDigest(HashAlgorithm.sha512));
-        default:
-            throw new IllegalArgumentException("unknown HashAlgorithm");
-        }
+        return new HMacOperator(createDigest(TlsUtils.getHashAlgorithmForHMACAlgorithm(macAlgorithm)));
     }
 
     public TlsSRP6Client createSRP6Client(TlsSRPConfig srpConfig)
@@ -740,94 +651,6 @@ public class BcTlsCrypto
         };
     }
 
-    protected TlsHMAC createSSl3HMAC(int macAlgorithm)
-        throws IOException
-    {
-        switch (macAlgorithm)
-        {
-        case MACAlgorithm._null:
-            return null;
-        case MACAlgorithm.hmac_md5:
-            return new SSL3Mac(createDigest(HashAlgorithm.md5));
-        case MACAlgorithm.hmac_sha1:
-            return new SSL3Mac(createDigest(HashAlgorithm.sha1));
-        case MACAlgorithm.hmac_sha256:
-            return new SSL3Mac(createDigest(HashAlgorithm.sha256));
-        case MACAlgorithm.hmac_sha384:
-            return new SSL3Mac(createDigest(HashAlgorithm.sha384));
-        case MACAlgorithm.hmac_sha512:
-            return new SSL3Mac(createDigest(HashAlgorithm.sha512));
-        default:
-            throw new TlsFatalAlert(AlertDescription.internal_error);
-        }
-    }
-
-    public class CombinedPRF
-        implements Digest
-    {
-        private final MD5Digest md5;
-        private final SHA1Digest sha1;
-
-        CombinedPRF()
-        {
-            this.md5 = new MD5Digest();
-            this.sha1 = new SHA1Digest();
-        }
-
-        /**
-         * @see org.bouncycastle.crypto.Digest#getAlgorithmName()
-         */
-        public String getAlgorithmName()
-        {
-            return md5.getAlgorithmName() + " and " + sha1.getAlgorithmName();
-        }
-
-        /**
-         * @see org.bouncycastle.crypto.Digest#getDigestSize()
-         */
-        public int getDigestSize()
-        {
-            return md5.getDigestSize() + sha1.getDigestSize();
-        }
-
-        /**
-         * @see org.bouncycastle.crypto.Digest#update(byte)
-         */
-        public void update(byte input)
-        {
-            md5.update(input);
-            sha1.update(input);
-        }
-
-        /**
-         * @see org.bouncycastle.crypto.Digest#update(byte[], int, int)
-         */
-        public void update(byte[] input, int inOff, int len)
-        {
-            md5.update(input, inOff, len);
-            sha1.update(input, inOff, len);
-        }
-
-        /**
-         * @see org.bouncycastle.crypto.Digest#doFinal(byte[], int)
-         */
-        public int doFinal(byte[] output, int outOff)
-        {
-            int i1 = md5.doFinal(output, outOff);
-            int i2 = sha1.doFinal(output, outOff + i1);
-            return i1 + i2;
-        }
-
-        /**
-         * @see org.bouncycastle.crypto.Digest#reset()
-         */
-        public void reset()
-        {
-            md5.reset();
-            sha1.reset();
-        }
-    }
-
     private class BlockOperator
         implements TlsBlockCipherImpl
     {
@@ -842,15 +665,15 @@ public class BcTlsCrypto
             this.isEncrypting = isEncrypting;
         }
 
-        public void setKey(byte[] key)
+        public void setKey(byte[] key, int keyOff, int keyLen)
         {
-            this.key = new KeyParameter(key);
+            this.key = new KeyParameter(key, keyOff, keyLen);
             cipher.init(isEncrypting, new ParametersWithIV(this.key, new byte[cipher.getBlockSize()]));
         }
 
-        public void init(byte[] iv)
+        public void init(byte[] iv, int ivOff, int ivLen)
         {
-            cipher.init(isEncrypting, new ParametersWithIV(null, iv));
+            cipher.init(isEncrypting, new ParametersWithIV(null, iv, ivOff, ivLen));
         }
 
         public int doFinal(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset)
@@ -871,43 +694,6 @@ public class BcTlsCrypto
         }
     }
 
-    private class StreamOperator
-        implements TlsStreamCipherImpl
-    {
-        private final boolean isEncrypting;
-        private final StreamCipher cipher;
-
-        private KeyParameter key;
-
-        StreamOperator(StreamCipher cipher, boolean isEncrypting)
-        {
-            this.cipher = cipher;
-            this.isEncrypting = isEncrypting;
-        }
-
-        public void setKey(byte[] key)
-        {
-            this.key = new KeyParameter(key);
-        }
-
-        public void init(byte[] iv)
-        {
-            if (iv != null)
-            {
-                cipher.init(isEncrypting, new ParametersWithIV(this.key, iv));
-            }
-            else
-            {
-                cipher.init(isEncrypting, this.key);
-            }
-        }
-
-        public int doFinal(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset)
-        {
-            return cipher.processBytes(input, inputOffset, inputLength, output, outputOffset);
-        }
-    }
-
     public class AeadOperator
         implements TlsAEADCipherImpl
     {
@@ -922,9 +708,9 @@ public class BcTlsCrypto
             this.isEncrypting = isEncrypting;
         }
 
-        public void setKey(byte[] key)
+        public void setKey(byte[] key, int keyOff, int keyLen)
         {
-            this.key = new KeyParameter(key);
+            this.key = new KeyParameter(key, keyOff, keyLen);
         }
 
         public void init(byte[] nonce, int macSize, byte[] additionalData)
@@ -948,50 +734,11 @@ public class BcTlsCrypto
             catch (InvalidCipherTextException e)
             {
                 // TODO:
-                e.printStackTrace(); throw new RuntimeCryptoException(e.toString());
+                throw new RuntimeCryptoException(e.toString());
             }
         }
     }
 
-    private class MacOperator implements TlsMAC
-    {
-        private final Mac mac;
-
-        MacOperator(Mac mac)
-        {
-            this.mac = mac;
-        }
-
-        public void setKey(byte[] key)
-        {
-            mac.init(new KeyParameter(key));
-        }
-
-        public void update(byte[] input, int inOff, int length)
-        {
-            mac.update(input, inOff, length);
-        }
-
-        public byte[] calculateMAC()
-        {
-            byte[] rv = new byte[mac.getMacSize()];
-
-            mac.doFinal(rv, 0);
-
-            return rv;
-        }
-
-        public int getMacLength()
-        {
-            return mac.getMacSize();
-        }
-
-        public void reset()
-        {
-            mac.reset();
-        }
-    }
-
     private class HMacOperator implements TlsHMAC
     {
         private final HMac hmac;
@@ -1001,9 +748,9 @@ public class BcTlsCrypto
             this.hmac = new HMac(digest);
         }
 
-        public void setKey(byte[] key)
+        public void setKey(byte[] key, int keyOff, int keyLen)
         {
-            hmac.init(new KeyParameter(key));
+            hmac.init(new KeyParameter(key, keyOff, keyLen));
         }
 
         public void update(byte[] input, int inOff, int length)
@@ -1035,11 +782,4 @@ public class BcTlsCrypto
             hmac.reset();
         }
     }
-
-    private static long counter = Times.nanoTime();
-
-    private synchronized static long nextCounterValue()
-    {
-        return ++counter;
-    }
 }
