@@ -3,49 +3,36 @@ package org.bouncycastle.tls.crypto.impl.bc;
 import java.io.IOException;
 
 import org.bouncycastle.crypto.CryptoException;
+import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.Signer;
-import org.bouncycastle.crypto.digests.NullDigest;
 import org.bouncycastle.crypto.encodings.PKCS1Encoding;
 import org.bouncycastle.crypto.engines.RSABlindedEngine;
-import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
 import org.bouncycastle.crypto.params.ParametersWithRandom;
+import org.bouncycastle.crypto.params.RSAKeyParameters;
 import org.bouncycastle.crypto.signers.GenericSigner;
 import org.bouncycastle.crypto.signers.RSADigestSigner;
 import org.bouncycastle.tls.AlertDescription;
+import org.bouncycastle.tls.HashAlgorithm;
 import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.SignatureAndHashAlgorithm;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsUtils;
-import org.bouncycastle.tls.crypto.TlsSigner;
-import org.bouncycastle.tls.crypto.TlsStreamSigner;
 
 /**
  * Operator supporting the generation of RSA signatures using the BC light-weight API.
  */
 public class BcTlsRSASigner
-    implements TlsSigner
+    extends BcTlsSigner
 {
-    private final AsymmetricKeyParameter privateKey;
-    private final BcTlsCrypto crypto;
-
-    public BcTlsRSASigner(BcTlsCrypto crypto, AsymmetricKeyParameter privateKey)
+    public BcTlsRSASigner(BcTlsCrypto crypto, RSAKeyParameters privateKey)
     {
-        this.crypto = crypto;
-        if (privateKey == null)
-        {
-            throw new IllegalArgumentException("'privateKey' cannot be null");
-        }
-        if (!privateKey.isPrivate())
-        {
-            throw new IllegalArgumentException("'privateKey' must be private");
-        }
-
-        this.privateKey = privateKey;
+        super(crypto, privateKey);
     }
 
-    public byte[] generateRawSignature(SignatureAndHashAlgorithm algorithm,
-                                       byte[] hash) throws IOException
+    public byte[] generateRawSignature(SignatureAndHashAlgorithm algorithm, byte[] hash) throws IOException
     {
+        Digest nullDigest = crypto.createDigest(HashAlgorithm.none);
+
         Signer signer;
         if (algorithm != null)
         {
@@ -58,7 +45,7 @@ public class BcTlsRSASigner
              * RFC 5246 4.7. In RSA signing, the opaque vector contains the signature generated
              * using the RSASSA-PKCS1-v1_5 signature scheme defined in [PKCS1].
              */
-            signer = new RSADigestSigner(new NullDigest(), TlsUtils.getOIDForHashAlgorithm(algorithm.getHash()));
+            signer = new RSADigestSigner(nullDigest, TlsUtils.getOIDForHashAlgorithm(algorithm.getHash()));
         }
         else
         {
@@ -66,7 +53,7 @@ public class BcTlsRSASigner
              * RFC 5246 4.7. Note that earlier versions of TLS used a different RSA signature scheme
              * that did not include a DigestInfo encoding.
              */
-            signer = new GenericSigner(new PKCS1Encoding(new RSABlindedEngine()), new NullDigest());
+            signer = new GenericSigner(new PKCS1Encoding(new RSABlindedEngine()), nullDigest);
         }
         signer.init(true, new ParametersWithRandom(privateKey, crypto.getSecureRandom()));
         signer.update(hash, 0, hash.length);
@@ -79,9 +66,4 @@ public class BcTlsRSASigner
             throw new TlsFatalAlert(AlertDescription.internal_error, e);
         }
     }
-
-    public TlsStreamSigner getStreamSigner(SignatureAndHashAlgorithm algorithm)
-    {
-        return null;
-    }
 }
