@@ -2,6 +2,7 @@ package org.bouncycastle.tls.crypto.impl.jcajce;
 
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
+import java.math.BigInteger;
 import java.security.GeneralSecurityException;
 import java.security.PublicKey;
 import java.security.cert.CertificateEncodingException;
@@ -12,6 +13,11 @@ import java.security.interfaces.RSAPublicKey;
 
 import javax.crypto.interfaces.DHPublicKey;
 
+import org.bouncycastle.asn1.ASN1Encoding;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.ASN1OctetString;
+import org.bouncycastle.asn1.ASN1Primitive;
+import org.bouncycastle.asn1.x509.Certificate;
 import org.bouncycastle.asn1.x509.Extensions;
 import org.bouncycastle.asn1.x509.KeyUsage;
 import org.bouncycastle.asn1.x509.TBSCertificate;
@@ -32,41 +38,61 @@ import org.bouncycastle.tls.crypto.TlsVerifier;
 public class JcaTlsCertificate
     implements TlsCertificate
 {
-    private final JcaJceHelper helper;
-
-    public static JcaTlsCertificate convert(TlsCertificate certificate, JcaJceHelper helper) throws IOException
+    public static JcaTlsCertificate convert(JcaTlsCrypto crypto, TlsCertificate certificate) throws IOException
     {
         if (certificate instanceof JcaTlsCertificate)
         {
             return (JcaTlsCertificate)certificate;
         }
 
-        return new JcaTlsCertificate(certificate.getEncoded(), helper);
+        return new JcaTlsCertificate(crypto, certificate.getEncoded());
     }
 
+    public static X509Certificate parseCertificate(JcaJceHelper helper, byte[] encoding)
+        throws IOException
+    {
+        try
+        {
+            /*
+             * NOTE: We want to restrict 'encoding' to a binary BER encoding, but
+             * CertificateFactory.generateCertificate claims to require DER encoding, and also
+             * supports Base64 encodings (in PEM format), which we don't support.
+             * 
+             * Re-encoding validates as BER and produces DER.
+             */
+            byte[] derEncoding = Certificate.getInstance(encoding).getEncoded(ASN1Encoding.DER);
+
+            ByteArrayInputStream input = new ByteArrayInputStream(derEncoding);
+            X509Certificate certificate = (X509Certificate)helper.createCertificateFactory("X.509").generateCertificate(input);
+            if (input.available() != 0)
+            {
+                throw new IOException("Extra data detected in stream");
+            }
+            return certificate;
+        }
+        catch (GeneralSecurityException e)
+        {
+            throw new TlsCryptoException("unable to decode certificate", e);
+        }
+    }
+
+    protected final JcaTlsCrypto crypto;
     protected final X509Certificate certificate;
 
     protected DHPublicKey pubKeyDH = null;
     protected ECPublicKey pubKeyEC = null;
     protected RSAPublicKey pubKeyRSA = null;
 
-    public JcaTlsCertificate(byte[] encoding, JcaJceHelper helper)
+    public JcaTlsCertificate(JcaTlsCrypto crypto, byte[] encoding)
         throws IOException
     {
-        try
-        {
-            ByteArrayInputStream input = new ByteArrayInputStream(encoding);
-            this.certificate = (X509Certificate)helper.createCertificateFactory("X.509").generateCertificate(input);
-            if (input.available() != 0)
-            {
-                throw new IOException("Extra data detected in stream");
-            }
-        }
-        catch (GeneralSecurityException e)
-        {
-            throw new TlsCryptoException("unable to decode certificate: " + e.getMessage(), e);
-        }
-        this.helper = helper;
+        this(crypto, parseCertificate(crypto.getHelper(), encoding));
+    }
+
+    public JcaTlsCertificate(JcaTlsCrypto crypto, X509Certificate certificate)
+    {
+        this.crypto = crypto;
+        this.certificate = certificate;
     }
 
     public TlsVerifier createVerifier(short signatureAlgorithm) throws IOException
@@ -76,13 +102,13 @@ public class JcaTlsCertificate
         switch (signatureAlgorithm)
         {
         case SignatureAlgorithm.dsa:
-            return new JcaTlsDSAVerifier(getPubKeyDSS(), helper);
+            return new JcaTlsDSAVerifier(getPubKeyDSS(), crypto.getHelper());
 
         case SignatureAlgorithm.ecdsa:
-            return new JcaTlsECDSAVerifier(getPubKeyEC(), helper);
+            return new JcaTlsECDSAVerifier(getPubKeyEC(), crypto.getHelper());
 
         case SignatureAlgorithm.rsa:
-            return new JcaTlsRSAVerifier(getPubKeyRSA(), helper);
+            return new JcaTlsRSAVerifier(getPubKeyRSA(), crypto.getHelper());
 
         default:
             throw new TlsFatalAlert(AlertDescription.certificate_unknown);
@@ -165,6 +191,17 @@ public class JcaTlsCertificate
         }
     }
 
+    public byte[] getExtension(ASN1ObjectIdentifier extensionOID) throws IOException
+    {
+        byte[] encoding = certificate.getExtensionValue(extensionOID.getId());
+        return encoding == null ? null : ((ASN1OctetString)ASN1Primitive.fromByteArray(encoding)).getOctets();
+    }
+
+    public BigInteger getSerialNumber()
+    {
+        return certificate.getSerialNumber();
+    }
+
     DHPublicKey getPubKeyDH() throws IOException
     {
         DHPublicKey pubKeyDH;
