@@ -1,7 +1,9 @@
 package org.bouncycastle.tls.crypto.impl.jcajce;
 
 import java.lang.reflect.Constructor;
+import java.security.AccessController;
 import java.security.GeneralSecurityException;
+import java.security.PrivilegedAction;
 import java.security.spec.AlgorithmParameterSpec;
 
 import javax.crypto.Cipher;
@@ -22,8 +24,21 @@ public class JceAEADCipherImpl
     {
         try
         {
-            Class<AlgorithmParameterSpec> clazz = (Class<AlgorithmParameterSpec>)
-                Class.forName("javax.crypto.spec.GCMParameterSpec", true, IvParameterSpec.class.getClassLoader());
+            Class<AlgorithmParameterSpec> clazz = AccessController.doPrivileged(new PrivilegedAction<Class<AlgorithmParameterSpec>>()
+            {
+                public Class<AlgorithmParameterSpec> run()
+                {
+                    try
+                    {
+                        return (Class<AlgorithmParameterSpec>)
+                            Class.forName("javax.crypto.spec.GCMParameterSpec", true, IvParameterSpec.class.getClassLoader());
+                    }
+                    catch (Exception e)
+                    {
+                        return null;
+                    }
+                }
+            });
             return clazz.getConstructor(int.class, byte[].class);
         }
         catch (Exception ignore)
@@ -49,9 +64,9 @@ public class JceAEADCipherImpl
         this.cipherMode = (isEncrypting) ? Cipher.ENCRYPT_MODE : Cipher.DECRYPT_MODE;
     }
 
-    public void setKey(byte[] key)
+    public void setKey(byte[] key, int keyOff, int keyLen)
     {
-        this.key = new SecretKeySpec(key, algorithm);
+        this.key = new SecretKeySpec(key, keyOff, keyLen, algorithm);
     }
 
     public void init(byte[] nonce, int macSize, byte[] additionalData)
