@@ -7,15 +7,18 @@ import java.security.KeyFactory;
 import java.security.KeyPair;
 import java.security.KeyPairGenerator;
 
-import javax.crypto.KeyAgreement;
+import javax.crypto.SecretKey;
 import javax.crypto.interfaces.DHPrivateKey;
 import javax.crypto.interfaces.DHPublicKey;
 import javax.crypto.spec.DHParameterSpec;
 import javax.crypto.spec.DHPublicKeySpec;
 
 import org.bouncycastle.tls.AlertDescription;
+import org.bouncycastle.tls.TlsDHUtils;
 import org.bouncycastle.tls.TlsFatalAlert;
+import org.bouncycastle.tls.crypto.DHGroup;
 import org.bouncycastle.tls.crypto.TlsAgreement;
+import org.bouncycastle.tls.crypto.TlsCryptoException;
 import org.bouncycastle.tls.crypto.TlsDHConfig;
 import org.bouncycastle.tls.crypto.TlsDHDomain;
 import org.bouncycastle.util.BigIntegers;
@@ -26,32 +29,49 @@ import org.bouncycastle.util.BigIntegers;
 public class JceTlsDHDomain
     implements TlsDHDomain
 {
-    protected JcaTlsCrypto crypto;
-    protected TlsDHConfig dhConfig;
-    protected DHParameterSpec dhDomain;
+    public static DHParameterSpec getParameters(TlsDHConfig dhConfig)
+    {
+        DHGroup dhGroup = TlsDHUtils.getDHGroup(dhConfig);
+        if (dhGroup == null)
+        {
+            throw new IllegalArgumentException("No DH configuration provided");
+        }
+
+        // NOTE: dhGroup.getQ() is ignored here
+        return new DHParameterSpec(dhGroup.getP(), dhGroup.getG(), dhGroup.getL());
+    }
+
+    protected final JcaTlsCrypto crypto;
+    protected final TlsDHConfig dhConfig;
+    protected final DHParameterSpec dhParameterSpec;
 
     public JceTlsDHDomain(JcaTlsCrypto crypto, TlsDHConfig dhConfig)
     {
         this.crypto = crypto;
         this.dhConfig = dhConfig;
-        this.dhDomain = getParameters(dhConfig);
+        this.dhParameterSpec = getParameters(dhConfig);
     }
 
-    public byte[] calculateDHAgreement(DHPublicKey publicKey, DHPrivateKey privateKey)
-        throws GeneralSecurityException
+    public JceTlsSecret calculateDHAgreement(DHPrivateKey privateKey, DHPublicKey publicKey)
+        throws IOException
     {
-        KeyAgreement agreement = crypto.getHelper().createKeyAgreement("DH");
+        try
+        {
+            /*
+             * RFC 5246 8.1.2. Leading bytes of Z that contain all zero bits are stripped before it is
+             * used as the pre_master_secret. We use the convention established by the JSSE to signal this
+             * by asking for "TlsPremasterSecret".
+             */
+            SecretKey secretKey = crypto.calculateKeyAgreement("DH", privateKey, publicKey, "TlsPremasterSecret");
 
-        agreement.init(privateKey);
+            // TODO Need to consider cases where SecretKey may not be encodable
 
-        agreement.doPhase(publicKey, true);
-
-        /*
-         * RFC 5246 8.1.2. Leading bytes of Z that contain all zero bits are stripped before it is
-         * used as the pre_master_secret. We use the convention established by the JSSE to signal this
-         * by asking for "TlsPremasterSecret".
-         */
-        return agreement.generateSecret("TlsPremasterSecret").getEncoded();
+            return crypto.adoptLocalSecret(secretKey.getEncoded());
+        }
+        catch (GeneralSecurityException e)
+        {
+            throw new TlsCryptoException("cannot calculate secret", e);
+        }
     }
 
     public TlsAgreement createDH()
@@ -66,18 +86,22 @@ public class JceTlsDHDomain
 
     public DHPublicKey decodePublicKey(byte[] encoding) throws IOException
     {
+        /*
+         * RFC 7919 3. [..] the client MUST verify that dh_Ys is in the range 1 < dh_Ys < dh_p - 1.
+         * If dh_Ys is not in this range, the client MUST terminate the connection with a fatal
+         * handshake_failure(40) alert.
+         */
         try
         {
             BigInteger y = decodeParameter(encoding);
 
-            // TODO Check RFCs for any validation that could/should be done here
             KeyFactory keyFactory = crypto.getHelper().createKeyFactory("DH");
 
-            return (DHPublicKey)keyFactory.generatePublic(new DHPublicKeySpec(y, dhDomain.getP(), dhDomain.getG()));
+            return (DHPublicKey)keyFactory.generatePublic(new DHPublicKeySpec(y, dhParameterSpec.getP(), dhParameterSpec.getG()));
         }
         catch (Exception e)
         {
-            throw new TlsFatalAlert(AlertDescription.illegal_parameter, e);
+            throw new TlsFatalAlert(AlertDescription.handshake_failure, e);
         }
     }
 
@@ -91,35 +115,17 @@ public class JceTlsDHDomain
         return encodeParameter(publicKey.getY());
     }
 
-    public KeyPair generateKeyPair()
+    public KeyPair generateKeyPair() throws IOException
     {
         try
         {
             KeyPairGenerator keyPairGenerator = crypto.getHelper().createKeyPairGenerator("DH");
-            keyPairGenerator.initialize(getParameters(dhConfig), crypto.getSecureRandom());
+            keyPairGenerator.initialize(dhParameterSpec, crypto.getSecureRandom());
             return keyPairGenerator.generateKeyPair();
         }
         catch (GeneralSecurityException e)
         {
-            throw new IllegalStateException("unable to create key pair: " + e.getMessage(), e);
+            throw new TlsCryptoException("unable to create key pair", e);
         }
     }
-
-    public JcaTlsCrypto getCrypto()
-    {
-        return crypto;
-    }
-
-    public DHParameterSpec getParameters(TlsDHConfig dhConfig)
-    {
-        // TODO There's a draft RFC for negotiated (named) groups
-
-        BigInteger[] pg = dhConfig.getExplicitPG();
-        if (pg != null)
-        {
-            return new DHParameterSpec(pg[0], pg[1]);
-        }
-
-        throw new IllegalStateException("No DH configuration provided");
-    }
 }
