@@ -25,6 +25,13 @@ public class AllTests
         suite.addTestSuite(InstanceTest.class);
         suite.addTestSuite(KeyManagerFactoryTest.class);
 
+        if (hasClass("javax.net.ssl.CertPathTrustManagerParameters"))
+        {
+            suite.addTestSuite(TrustManagerFactoryTest.class);
+        }
+
+        suite.addTest(CipherSuitesTestSuite.suite());
+
         return new BCTestSetup(suite);
     }
 
@@ -46,4 +53,17 @@ public class AllTests
 
         }
     }
+
+    private static boolean hasClass(String name)
+    {
+        try
+        {
+            AllTests.class.getClassLoader().loadClass(name);
+            return true;
+        }
+        catch (Exception e)
+        {
+            return false;
+        }
+    }
 }
