@@ -26,6 +26,7 @@ import org.bouncycastle.tls.TlsAuthentication;
 import org.bouncycastle.tls.TlsCredentialedSigner;
 import org.bouncycastle.tls.TlsCredentials;
 import org.bouncycastle.tls.TlsFatalAlert;
+import org.bouncycastle.tls.TlsServerCertificate;
 import org.bouncycastle.tls.TlsUtils;
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.TlsCrypto;
@@ -176,10 +177,10 @@ class TlsTestClientImpl
     {
         return new TlsAuthentication()
         {
-            public void notifyServerCertificate(org.bouncycastle.tls.Certificate serverCertificate)
+            public void notifyServerCertificate(TlsServerCertificate serverCertificate)
                 throws IOException
             {
-                TlsCertificate[] chain = serverCertificate.getCertificateList();
+                TlsCertificate[] chain = serverCertificate.getCertificate().getCertificateList();
 
                 if (TlsTestConfig.DEBUG)
                 {
@@ -193,9 +194,10 @@ class TlsTestClientImpl
                     }
                 }
 
-                boolean isEmpty = serverCertificate == null || serverCertificate.isEmpty();
+                boolean isEmpty = serverCertificate == null || serverCertificate.getCertificate() == null
+                    || serverCertificate.getCertificate().isEmpty();
                 if (isEmpty || !TlsTestUtils.isCertificateOneOf(context.getCrypto(), chain[0],
-                    new String[]{ "x509-server.pem", "x509-server-dsa.pem", "x509-server-ecdsa.pem"}))
+                    new String[]{ "x509-server-dsa.pem", "x509-server-ecdsa.pem", "x509-server-rsa-enc.pem", "x509-server-rsa-sign.pem" }))
                 {
                     throw new TlsFatalAlert(AlertDescription.bad_certificate);
                 }
@@ -227,7 +229,7 @@ class TlsTestClientImpl
                 }
 
                 final TlsCredentialedSigner signerCredentials = TlsTestUtils.loadSignerCredentials(context,
-                    supportedSigAlgs, SignatureAlgorithm.rsa, "x509-client.pem", "x509-client-key.pem");
+                    supportedSigAlgs, SignatureAlgorithm.rsa, "x509-client-rsa.pem", "x509-client-key-rsa.pem");
 
                 if (config.clientAuth == TlsTestConfig.CLIENT_AUTH_VALID)
                 {
